@@ -1,6 +1,6 @@
 /*
  * Algorithms.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com) and Eric Tamme,  2009-2011.
+ * (rocketman768@gmail.com) and Eric Tamme,  2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #include <cmath>
 #include <math.h>
 #include "Algorithms.h"
+#include "PhysicalConstants.h"
 
 // Called when Instance() is called, should only initialize once.
 Algorithms::Algorithms()
@@ -41,16 +42,6 @@ Algorithms::Algorithms()
    hydroCorrection15CPoly[1] = -16.2853e-3;
    hydroCorrection15CPoly[2] = 5.84346e-3;
    hydroCorrection15CPoly[3] = -15.3243e-6;
-
-   sucroseDensity_kgL = 1.587;
-}
-
-bool Algorithms::isnan(double d)
-{
-   // If using IEEE floating points, all comparisons with a NaN
-   // are false, so the following should be true only if we have
-   // a NaN.
-   return (d != d);
 }
 
 double Algorithms::round(double d)
@@ -67,7 +58,6 @@ inline double Algorithms::intPow( double base, unsigned int pow )
    return ret;
 }
 
-// NOTE: order is the ORDER of the polynomial, NOT THE SIZE of the poly array.
 double Algorithms::polyEval( double* poly, unsigned int order, double x )
 {
    double ret = 0.0;
@@ -81,11 +71,10 @@ double Algorithms::polyEval( double* poly, unsigned int order, double x )
    return ret;
 }
 
-// Root finding of a polynomial via the secant method. Returns HUGE_VAL on failure.
 double Algorithms::rootFind( double* poly, unsigned int order, double x0, double x1 )
 {
    double guesses[] = { x0, x1 };
-   double newGuess;
+   double newGuess = x0;
    double maxAllowableSeparation = qAbs( x0 - x1 ) * 1e3;
 
    while( qAbs( guesses[0] - guesses[1] ) > ROOT_PRECISION )
@@ -102,7 +91,6 @@ double Algorithms::rootFind( double* poly, unsigned int order, double x0, double
    return newGuess;
 }
 
-// Returns the additive correction (in SG 15C units).
 double Algorithms::hydrometer15CCorrection( double celsius )
 {
    return polyEval( hydroCorrection15CPoly,hydroCorrection15CPoly_order, celsius ) * (double)1e-3;
@@ -135,7 +123,7 @@ double Algorithms::PlatoToSG_20C20C( double plato )
 
 double Algorithms::getPlato( double sugar_kg, double wort_l )
 {
-   double water_kg = wort_l - sugar_kg/sucroseDensity_kgL; // Assumes sucrose vol and water vol add to wort vol.
+   double water_kg = wort_l - sugar_kg/PhysicalConstants::sucroseDensity_kgL; // Assumes sucrose vol and water vol add to wort vol.
 
    return sugar_kg/(sugar_kg+water_kg) * 100.0;
 }
