@@ -1,6 +1,6 @@
 /*
  * Algorithms.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com) and Eric Tamme,  2009-2011.
+ * (rocketman768@gmail.com) and Eric Tamme,  2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,15 @@
 #define ROOT_PRECISION 0.0000001
 
 #include <QList>
+#include <limits> // For std::numeric_limits
 
+/*!
+ * \class Algorithms
+ * \author Philip G. Lee
+ * \author Eric Tamme
+ *
+ * \brief Singleton that holds some beer-related math functions as well as some arithmetic and CS algorithms.
+ */
 class Algorithms
 {
 public:
@@ -31,8 +39,25 @@ public:
      return algorithmsSingleton;
    }
 
-   // Need the following to be cross-platform compatible.
-   bool isnan(double d);
+   //! Cross-platform compatible NaN checker.
+   inline bool isnan(double d)
+   {
+      // If using IEEE floating points, all comparisons with a NaN
+      // are false, so the following should be true only if we have
+      // a NaN.
+      return (d != d);
+   }
+   //! Cross-platform compatible Inf checker.
+   template<typename T> inline bool isinf(T var)
+   {
+      return
+      (
+         std::numeric_limits<T>::has_infinity &&
+         var == std::numeric_limits<T>::infinity()
+         //(var < std::numeric_limits<T>::min() || var > std::numeric_limits<T>::max())
+      );
+   }
+   //! Cross-platform rounding.
    double round(double d);
 
    /**
@@ -43,7 +68,7 @@ public:
     * Why isn't this already a member of QList?
     */
    template<class T> void unDup( QList<T>& list, bool (*equals)(T,T) )
-   { // Must be implemented in this header file apparently :(
+   {
       int i = 0;
 
       while( i < list.size() - 1 )
@@ -55,32 +80,54 @@ public:
       }
    }
 
-   // Polynomial evaluation
+   //! \returns base^pow
    double intPow( double base, unsigned int pow );
+   
+   /*!
+    * Evaluates a finite-order polynomial.
+    * \param x - where to evaluate f(x)
+    * \param order - the order of the polynomial, NOT the size of the \c poly array.
+    * \param poly - f(x) = poly[0]x^0 + poly[1]x^1 + ... + poly[order]x^order
+    */
    double polyEval( double* poly, unsigned int order, double x );
-   // Root finding by the secant method. Returns HUGE_VAL on failure.
+   
+   /*!
+    * Root-finding by the secant method.
+    * \param poly - see \b polyEval
+    * \param order - order of the polynomial
+    * \param x0 - one of two initial DISTINCT guesses at the root
+    * \param x1 - one of two initial DISTINCT guesses at the root
+    * \returns HUGE_VAL on failure, otherwise a root of the polynomial
+    */
    double rootFind( double* poly, unsigned int order, double x0, double x1 );
 
    //===================Beer-related stuff=====================
+   
+   //! \returns plato of \b sg
    double SG_20C20C_toPlato( double sg );
+   //! \returns sg of \b plato
    double PlatoToSG_20C20C( double plato );
+   //! \returns water density in kg/L at temperature \b celsius
    double getWaterDensity_kgL( double celsius );
+   //! \returns additive correction to the 15C hydrometer reading if read at \b celsius
    double hydrometer15CCorrection( double celsius );
 
-   // Estimates plato from kg of dissolved sucrose (sugar_kg) and
-   // the total wort volume wort_l.
+   /*!
+    * Estimates plato from kg of dissolved sucrose (sugar_kg) and
+    * the total wort volume wort_l.
+    */
    double getPlato( double sugar_kg, double wort_l );
-   // Converts FG to plato, given the OG.
+   //! Converts FG to plato, given the OG.
    double ogFgToPlato( double og, double fg );
-   // Gets ABV by using current gravity reading and brix reading.
+   //! Gets ABV by using current gravity reading and brix reading.
    double getABVBySGPlato( double sg, double plato );
-   // Gets ABW from current gravity and plato.
+   //! Gets ABW from current gravity and plato.
    double getABWBySGPlato( double sg, double plato );
-   // Gives you the SG from the starting plato and current plato.
+   //! Gives you the SG from the starting plato and current plato.
    double sgByStartingPlato( double startingPlato, double currentPlato );
-   // Returns the refractive index from plato.
+   //! Returns the refractive index from plato.
    double refractiveIndex( double plato );
-   // Corrects the apparent extract 'plato' to the real extract using current gravity 'sg'.
+   //! Corrects the apparent extract 'plato' to the real extract using current gravity 'sg'.
    double realExtract( double sg, double plato );
 
 private:
@@ -95,7 +142,6 @@ private:
    int waterDensityPoly_C_order;
    double hydroCorrection15CPoly[4];
    int hydroCorrection15CPoly_order;
-   double sucroseDensity_kgL;
 
    Algorithms();                         // ctor hidden
    Algorithms(Algorithms const&);             // copy ctor hidden
