@@ -1,6 +1,6 @@
 /*
  * BeerColorWidget.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,18 +25,30 @@ class BeerColorWidget;
 #include <QColor>
 #include <QPaintEvent>
 #include <QImage>
-#include "observable.h"
+#include <QMetaProperty>
+#include <QVariant>
 #include "recipe.h"
 
-class BeerColorWidget : public QWidget, public Observer
+/*!
+ * \class BeerColorWidget
+ * \author Philip G. Lee
+ *
+ * \brief Displays the approximate color of the beer on screen.
+ */
+class BeerColorWidget : public QWidget
 {
    Q_OBJECT
 
 public:
    BeerColorWidget(QWidget* parent=0);
-   virtual ~BeerColorWidget() {}
+
    void setColor( QColor newColor );
+   //! Observe \b rec for changes in color, and automatically update.
    void setRecipe( Recipe* rec );
+   
+public slots:
+   void parseChanges(QMetaProperty prop, QVariant val);
+   
 protected:
    virtual void paintEvent(QPaintEvent *);
    QColor color;
@@ -44,8 +56,6 @@ private:
    QImage glass;
    void showColor();
    
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer
-   
    Recipe* recObs;
 };
 
