@@ -1,6 +1,6 @@
 /*
  * BrewDayScrollWidget.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include "instruction.h"
 #include "brewtarget.h"
 #include "BrewDayScrollWidget.h"
+#include "database.h"
 #include <QListWidgetItem>
 #include <QPrinter>
 #include <QPrintDialog>
@@ -27,29 +28,30 @@
 #include <QDir>
 #include "InstructionWidget.h"
 #include "TimerWidget.h"
+#include "style.h"
+#include "equipment.h"
+#include "mash.h"
 
-BrewDayScrollWidget::BrewDayScrollWidget(QWidget* parent) : QWidget(parent), Observer()
+BrewDayScrollWidget::BrewDayScrollWidget(QWidget* parent)
+   : QWidget(parent), doc(new QWebView())
 {
    setupUi(this);
+   setObjectName("BrewDayScrollWidget");
    recObs = 0;
 
    connect( listWidget, SIGNAL(currentRowChanged(int)), this, SLOT(showInstruction(int)) );
-   connect( plainTextEdit, SIGNAL(textChanged()), this, SLOT(saveInstruction()) );
+   // connect( plainTextEdit, SIGNAL(textChanged()), this, SLOT(saveInstruction()) );
+   connect(btTextEdit,SIGNAL(textModified()), this, SLOT(saveInstruction()));
    connect( pushButton_insert, SIGNAL(clicked()), this, SLOT(insertInstruction()) );
    connect( pushButton_remove, SIGNAL(clicked()), this, SLOT(removeSelectedInstruction()) );
    connect( pushButton_up, SIGNAL(clicked()), this, SLOT(pushInstructionUp()) );
    connect( pushButton_down, SIGNAL(clicked()), this, SLOT(pushInstructionDown()) );
    connect( pushButton_generateInstructions, SIGNAL(clicked()), this, SLOT(generateInstructions()) );
-
-   doc = new QWebView();
 }
 
 void BrewDayScrollWidget::saveInstruction()
 {
-   // Need to disable notification to avoid a possible infinite loop.
-   recObs->disableNotification();
-   recObs->getInstruction( listWidget->currentRow() )->setDirections( plainTextEdit->toPlainText() );
-   recObs->reenableNotification();
+   recObs->instructions()[ listWidget->currentRow() ]->setDirections( btTextEdit->toPlainText() );
 }
 
 void BrewDayScrollWidget::showInstruction(int insNdx)
@@ -57,11 +59,12 @@ void BrewDayScrollWidget::showInstruction(int insNdx)
    if( recObs == 0 )
       return;
 
-   int size = recObs->getNumInstructions();
+   int size = recIns.size();
    if( insNdx < 0 || insNdx >= size )
       return;
 
-   plainTextEdit->setPlainText(recObs->getInstruction(insNdx)->getDirections());
+   // Block signals to avoid setPlainText() from triggering saveInstruction().
+   btTextEdit->setPlainText((recIns[insNdx])->directions());
 }
 
 void BrewDayScrollWidget::generateInstructions()
@@ -85,7 +88,7 @@ void BrewDayScrollWidget::removeSelectedInstruction()
    int row = listWidget->currentRow();
    if( row < 0 )
       return;
-   recObs->removeInstruction(recObs->getInstruction(row));
+   Database::instance().removeFromRecipe(recObs, recIns[row]);
 }
 
 void BrewDayScrollWidget::pushInstructionUp()
@@ -97,7 +100,7 @@ void BrewDayScrollWidget::pushInstructionUp()
    if( row <= 0 )
       return;
    
-   recObs->swapInstructions(row, row-1);
+   recObs->swapInstructions(recIns[row], recIns[row-1]);
    listWidget->setCurrentRow(row-1);
 }
 
@@ -108,18 +111,159 @@ void BrewDayScrollWidget::pushInstructionDown()
    
    int row = listWidget->currentRow();
 
-   if( row >= listWidget->count() )
-      return;
-
-   // This happens when the user is bouncing on the Step Down button but no
-   // instruction is selected.
-   if ( row < 0 )
+   if( row >= listWidget->count() || row < 0 )
       return;
    
-   recObs->swapInstructions(row, row+1);
+   recObs->swapInstructions(recIns[row], recIns[row+1]);
    listWidget->setCurrentRow(row+1);
 }
 
+bool BrewDayScrollWidget::loadComplete(bool ok) 
+{
+   doc->print(printer);
+   disconnect( doc, SIGNAL(loadFinished(bool)), this, SLOT(loadComplete(bool)) );
+   return ok;
+}
+
+void BrewDayScrollWidget::print(QPrinter *mainPrinter, QPrintDialog* dialog,
+      int action, QFile* outFile)
+{
+   QString pDoc;
+
+   if( recObs == 0 )
+      return;
+
+   /* Connect the webview's signal */
+   if ( action == PRINT )
+   {
+      printer = mainPrinter;
+      connect( doc, SIGNAL(loadFinished(bool)), this, SLOT(loadComplete(bool)) );
+
+      dialog->setWindowTitle(tr("Print Document"));
+      if (dialog->exec() != QDialog::Accepted)
+         return;
+   }
+
+   // Start building the document to be printed.  The HTML doesn't work with
+   // the image since it is a compiled resource
+   pDoc = buildTitleTable( action != HTML );
+   pDoc += buildInstructionTable();
+   pDoc += buildFooterTable();
+
+   pDoc += tr("<h2>Notes</h2>");
+   if ( recObs->notes() != "" )
+      pDoc += QString("%1").arg(recObs->notes());
+
+   pDoc += "</body></html>";
+
+   doc->setHtml(pDoc);
+   if ( action == PREVIEW )
+      doc->show();
+   else if ( action == HTML )
+   {
+      QTextStream out(outFile);
+      out << pDoc;
+      outFile->close();
+   }
+}
+
+void BrewDayScrollWidget::setRecipe(Recipe* rec)
+{
+   // Disconnect old notifier.
+   if( recObs )
+      disconnect( recObs, SIGNAL(changed(QMetaProperty,QVariant)), this, SLOT(acceptChanges(QMetaProperty,QVariant)) );
+   
+   recObs = rec;
+   connect( recObs, SIGNAL(changed(QMetaProperty,QVariant)), this, SLOT(acceptChanges(QMetaProperty,QVariant)) );
+   
+   recIns = recObs->instructions();
+   foreach( Instruction* ins, recIns )
+         connect( ins, SIGNAL(changed(QMetaProperty,QVariant)), this, SLOT(acceptInsChanges(QMetaProperty,QVariant)) );
+   
+   showChanges();
+}
+
+void BrewDayScrollWidget::insertInstruction()
+{
+   if( recObs == 0 )
+      return;
+
+   int pos = lineEdit_step->text().toInt();
+   Instruction* ins = Database::instance().newInstruction(recObs);
+
+   pos = qBound(1, pos, recIns.size());
+
+   ins->setName(lineEdit_name->text());
+
+   recObs->insertInstruction( ins, pos );
+   listWidget->setCurrentRow(pos-1);
+}
+
+void BrewDayScrollWidget::acceptChanges(QMetaProperty prop, QVariant /*value*/)
+{
+   if( recObs && QString(prop.name()) == "instructions" )
+   {
+      // An instruction has been added or deleted, so update internal list.
+      foreach( Instruction* ins, recIns )
+         disconnect( ins, 0, this, 0 );
+      recIns = recObs->instructions(); // Already sorted by instruction numbers.
+      foreach( Instruction* ins, recIns )
+         connect( ins, SIGNAL(changed(QMetaProperty,QVariant)), this, SLOT(acceptInsChanges(QMetaProperty,QVariant)) );
+      showChanges();
+   }
+}
+
+void BrewDayScrollWidget::acceptInsChanges(QMetaProperty prop, QVariant /*value*/)
+{
+   QString propName = prop.name();
+   
+   if( propName == "instructionNumber" )
+   {
+      // The order changed, so resort our internal list.
+      qSort( recIns.begin(), recIns.end(), insPtrLtByNumber );
+      showChanges();
+   }
+   else if( propName == "directions" )
+   {
+      // This will make the displayed text directions update.
+      listWidget->setCurrentRow( listWidget->currentRow() );
+   }
+}
+
+void BrewDayScrollWidget::clear()
+{
+   listWidget->clear();
+}
+
+void BrewDayScrollWidget::showChanges()
+{
+   clear();
+   if( recObs == 0 )
+      return;
+
+   repopulateListWidget();
+}
+
+void BrewDayScrollWidget::repopulateListWidget()
+{
+   listWidget->clear();
+
+   if( recObs == 0 )
+      return;
+   
+   foreach( Instruction* ins, recIns )
+   {
+      //QString text = tr("Step %1: %2").arg(i).arg(ins->name());
+      QString text = tr("Step %1: %2").arg(ins->instructionNumber()).arg(ins->name());
+      listWidget->addItem(new QListWidgetItem(text));
+   }
+
+   if( recIns.size() > 0 )
+      listWidget->setCurrentRow(0);
+   else
+      listWidget->setCurrentRow(-1);
+}
+
 QString BrewDayScrollWidget::getCSS() 
 {
    if ( cssName == NULL )
@@ -149,7 +293,7 @@ QString BrewDayScrollWidget::buildTitleTable(bool includeImage)
    header += "</style></head>";
 
    body   = "<body>";
-   body += QString("<h1>%1</h1>").arg(recObs->getName());
+   body += QString("<h1>%1</h1>").arg(recObs->name());
    if ( includeImage )
       body += QString("<img src=\"%1\" />").arg("qrc:/images/title.svg");
 
@@ -159,7 +303,7 @@ QString BrewDayScrollWidget::buildTitleTable(bool includeImage)
    body += QString("<tr><td class=\"left\">%1</td>")
          .arg(tr("Style"));
    body += QString("<td class=\"value\">%1</td>")
-           .arg(recObs->getStyle()->getName());
+           .arg(recObs->style()->name());
    body += QString("<td class=\"right\">%1</td>")
          .arg(tr("Date"));
    body += QString("<td class=\"value\">%1</td></tr>")
@@ -168,37 +312,37 @@ QString BrewDayScrollWidget::buildTitleTable(bool includeImage)
    // second row:  boil time and efficiency.  
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
             .arg(tr("Boil Time"))
-            .arg(Brewtarget::displayAmount(recObs->getEquipment()->getBoilTime_min(),Units::minutes))
+            .arg(Brewtarget::displayAmount(recObs->equipment()->boilTime_min(),Units::minutes))
             .arg(tr("Efficiency"))
-            .arg(Brewtarget::displayAmount(recObs->getEfficiency_pct(),0,0));
+            .arg(Brewtarget::displayAmount(recObs->efficiency_pct(),0,0));
 
    // third row: pre-Boil Volume and Preboil Gravity
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
             .arg(tr("Boil Volume"))
-            .arg(Brewtarget::displayAmount(recObs->estimateBoilVolume_l(),Units::liters,2))
+            .arg(Brewtarget::displayAmount(recObs->boilVolume_l(),Units::liters,2))
             .arg(tr("Preboil Gravity"))
-            .arg(Brewtarget::displayOG(recObs->getBoilGrav()));
+            .arg(Brewtarget::displayOG(recObs->boilGrav()));
 
    // fourth row: Final volume and starting gravity
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
             .arg(tr("Final Volume"))
-            .arg(Brewtarget::displayAmount(recObs->estimateFinalVolume_l(), Units::liters,2))
+            .arg(Brewtarget::displayAmount(recObs->finalVolume_l(), Units::liters,2))
             .arg(tr("Starting Gravity"))
-            .arg(Brewtarget::displayOG(recObs->getOg(), true));
+            .arg(Brewtarget::displayOG(recObs->og(), noUnit, true));
 
    // fifth row: IBU and Final gravity
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</tr>")
             .arg(tr("IBU"))
-            .arg( recObs->getIBU(),0,'f',1)
+            .arg( Brewtarget::displayAmount(recObs->IBU(),0,1))
             .arg(tr("Final Gravity"))
-            .arg(Brewtarget::displayFG(recObs->getFg(), recObs->getOg(), true));
+            .arg(Brewtarget::displayFG(recObs->fg(), recObs->og(), noUnit, true));
 
    // sixth row: ABV and estimate calories
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2%</td><td class=\"right\">%3</td><td class=\"value\">%4</tr>")
             .arg(tr("ABV"))
-            .arg( recObs->getABV_pct(), 0, 'f', 1)
+            .arg( Brewtarget::displayAmount(recObs->ABV_pct(),0,1) )
             .arg(tr("Estimated calories(per 12 oz)"))
-            .arg( recObs->estimateCalories(), 0, 'f', 0);
+            .arg( Brewtarget::displayAmount(recObs->calories(),0,0) );
    body += "</table>";
 
    return header + body;
@@ -217,33 +361,31 @@ QString BrewDayScrollWidget::buildInstructionTable()
          .arg(tr("Time"))
          .arg(tr("Step"));
 
-   size = recObs->getNumInstructions();
+   QList<Instruction*> instructions = recObs->instructions();
+   QList<MashStep*> mashSteps = recObs->mash()->mashSteps();
+   size = instructions.size();
    for( i = 0; i < size; ++i )
    {
       QString stepTime, tmp;
-      QVector<QString> reagents;
-      Instruction* ins = recObs->getInstruction(i);
+      QList<QString> reagents;
+      
+      Instruction* ins = instructions[i];
 
-      if (ins->getInterval())
-         stepTime = Brewtarget::displayAmount(ins->getInterval(), Units::minutes, 0);
+      if (ins->interval())
+         stepTime = Brewtarget::displayAmount(ins->interval(), Units::minutes, 0);
       else
          stepTime = "--";
 
       tmp = "";
-      reagents = ins->getReagents();
 
-      if ( ins->getName() == "Add grains")
-      {
-          Instruction* temp = recObs->getMashFermentable();
-          reagents = temp->getReagents();
-      }
-      else if ( ins->getName() == "Heat water")
-      {
-         int mashSize = recObs->getMash()->getNumMashSteps();
-         Instruction* temp = recObs->getMashWater(mashSize);
-         reagents = temp->getReagents();
-
-      }
+      // TODO: comparing ins->name() with these untranslated strings means this
+      // doesn't work in other languages. Find a better way.
+      if ( ins->name() == tr("Add grains") )
+         reagents = recObs->getReagents( recObs->fermentables() );
+      else if ( ins->name() == tr("Heat water") )
+         reagents = recObs->getReagents( recObs->mash()->mashSteps() );
+      else 
+         reagents = ins->reagents();
 
       if ( reagents.size() > 1 )
       {
@@ -251,13 +393,17 @@ QString BrewDayScrollWidget::buildInstructionTable()
          for ( j = 0; j < reagents.size(); j++ )
          {
             tmp += QString("<li>%1</li>")
-                   .arg(reagents[j]);
+                   .arg(reagents.at(j));
          }
          tmp += QString("</ul>");
       }
+      else if ( reagents.size() == 1 )
+      {
+         tmp = reagents.at(0);
+      }
       else
       {
-         tmp = reagents[0];
+         tmp = ins->directions();
       }
 
       QString altTag = i % 2 ? "alt" : "norm";
@@ -265,7 +411,7 @@ QString BrewDayScrollWidget::buildInstructionTable()
       middle += QString("<tr class=\"%1\"><td class=\"check\"></td><td class=\"time\">%2</td><td align=\"step\">%3 : %4</td></tr>")
                .arg(altTag)
                .arg(stepTime)
-               .arg(ins->getName())
+               .arg(ins->name())
                .arg(tmp);
    }
    middle += "</table>";
@@ -292,123 +438,3 @@ QString BrewDayScrollWidget::buildFooterTable()
 
    return bottom;
 }
-
-bool BrewDayScrollWidget::loadComplete(bool ok) 
-{
-   doc->print(printer);
-   disconnect( doc, SIGNAL(loadFinished(bool)), this, SLOT(loadComplete(bool)) );
-   return ok;
-}
-
-void BrewDayScrollWidget::print(QPrinter *mainPrinter, QPrintDialog* dialog,
-      int action, QFile* outFile)
-{
-   QString pDoc;
-//   QPrintDialog *dialog = new QPrintDialog(printer, this);
-
-   if( recObs == 0 )
-      return;
-
-   /* Connect the webview's signal */
-   if ( action == PRINT )
-   {
-      printer = mainPrinter;
-      connect( doc, SIGNAL(loadFinished(bool)), this, SLOT(loadComplete(bool)) );
-
-      dialog->setWindowTitle(tr("Print Document"));
-      if (dialog->exec() != QDialog::Accepted)
-         return;
-   }
-
-   // Start building the document to be printed.  The HTML doesn't work with
-   // the image since it is a compiled resource
-   pDoc = buildTitleTable( action != HTML );
-   pDoc += buildInstructionTable();
-   pDoc += buildFooterTable();
-
-   pDoc += tr("<h2>Notes</h2>");
-   if ( recObs->getNotes() != "" )
-      pDoc += QString("%1").arg(recObs->getNotes());
-
-   pDoc += "</body></html>";
-
-   doc->setHtml(pDoc);
-   if ( action == PREVIEW )
-      doc->show();
-   else if ( action == HTML )
-   {
-      QTextStream out(outFile);
-      out << pDoc;
-      outFile->close();
-   }
-}
-
-void BrewDayScrollWidget::setRecipe(Recipe* rec)
-{
-   recObs = rec;
-   setObserved(recObs);
-   showChanges();
-}
-
-void BrewDayScrollWidget::insertInstruction()
-{
-   if( recObs == 0 )
-      return;
-
-   int pos = lineEdit_step->text().toInt();
-   Instruction* ins = new Instruction();
-
-   if( pos < 0 || pos > recObs->getNumInstructions() )
-      pos = recObs->getNumInstructions();
-
-   ins->setName(lineEdit_name->text());
-
-   recObs->insertInstruction( ins, pos );
-}
-
-void BrewDayScrollWidget::notify(Observable* notifier, QVariant info)
-{
-   /*
-   if( notifier != recObs || info.toInt() != Recipe::INSTRUCTION )
-      return;
-   */
-
-   if( notifier == recObs && info.toInt() == Recipe::INSTRUCTION )
-      showChanges();
-}
-
-void BrewDayScrollWidget::clear()
-{
-   listWidget->clear();
-}
-
-void BrewDayScrollWidget::showChanges()
-{
-   clear();
-   if( recObs == 0 )
-      return;
-
-   repopulateListWidget();
-}
-
-void BrewDayScrollWidget::repopulateListWidget()
-{
-   listWidget->clear();
-
-   if( recObs == 0 )
-      return;
-
-   int i, size;
-   size = recObs->getNumInstructions();
-
-   for( i = 0; i < size; ++i )
-   {
-      QString text = tr("Step %1: %2").arg(i).arg(recObs->getInstruction(i)->getName());
-      listWidget->addItem(new QListWidgetItem(text));
-   }
-
-   if( size > 0 )
-      listWidget->setCurrentRow(0);
-   else
-      listWidget->setCurrentRow(-1);
-}
