@@ -1,6 +1,6 @@
 /*
  * BrewDayScrollWidget.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,10 +28,15 @@ class BrewDayScrollWidget;
 #include <QPrinter>
 #include <QPrintDialog>
 #include <QFile>
-#include "observable.h"
 #include "recipe.h"
 
-class BrewDayScrollWidget : public QWidget, public Ui::brewDayScrollWidget, public Observer
+/*!
+ * \class BrewDayScrollWidget
+ * \author Philip G. Lee
+ *
+ * \brief Widget that displays the brewday info in a scrollable area.
+ */
+class BrewDayScrollWidget : public QWidget, public Ui::brewDayScrollWidget
 {
    Q_OBJECT
 public:
@@ -39,15 +44,14 @@ public:
 
    BrewDayScrollWidget(QWidget* parent=0);
    virtual ~BrewDayScrollWidget() {}
-   //! Sets the observed recipe.
+   //! \brief Sets the observed recipe.
    void setRecipe(Recipe* rec);
 
    virtual QSize sizeHint() const; // From QWidget
-   virtual void notify(Observable *notifier, QVariant info); // From Observer.
 
    /*!
-    * Prints a paper version of the info in this dialog.
-    * Should probably be moved to its own view class.
+    *  \brief Prints a paper version of the info in this dialog.
+    *  Should be moved to its own view class.
     */
    void print(QPrinter* mainPrinter, QPrintDialog *dialog, int action = PRINT, QFile* outFile = 0);
 
@@ -63,12 +67,19 @@ public slots:
    //! Push selected instruction down.
    void pushInstructionDown();
 
+private slots:
+   //! \brief Receive notifications from the recipe.
+   void acceptChanges( QMetaProperty prop, QVariant value );
+   //! \brief Receive changes from instructions.
+   void acceptInsChanges( QMetaProperty prop, QVariant value );
+   
 private:
    //! Update the view.
    void showChanges();
    //! Repopulate the list widget with all the instructions.
    void repopulateListWidget();
    void clear();
+   
    QString buildTitleTable(bool includeImage = true);
    QString buildInstructionTable();
    QString buildFooterTable();
@@ -77,6 +88,8 @@ private:
    Recipe* recObs;
    QPrinter* printer;
    QWebView* doc;
+   //! Internal list of recipe instructions, always sorted by instruction number.
+   QList<Instruction*> recIns;
 
    QString cssName;
 
