@@ -1,6 +1,6 @@
 /*
  * BrewDayWidget.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,10 +25,19 @@ class BrewDayWidget;
 #include <QWidget>
 #include <QSize>
 #include <QWebView>
-#include "observable.h"
-#include "recipe.h"
+#include <QMetaProperty>
+#include <QVariant>
 
-class BrewDayWidget : public QWidget, public Ui::brewDayWidget, public Observer
+// Forward declarations.
+class Recipe;
+
+/*!
+ * \class BrewDayWidget
+ * \author Philip G. Lee
+ *
+ * \brief Widget that displays the brewday info. Deprecated I believe.
+ */
+class BrewDayWidget : public QWidget, public Ui::brewDayWidget
 {
    Q_OBJECT
 public:
@@ -37,7 +46,6 @@ public:
    void setRecipe(Recipe* rec);
 
    virtual QSize sizeHint() const; // From QWidget
-   virtual void notify(Observable *notifier, QVariant info); // From Observer.
 
 public slots:
    void insertInstruction();
@@ -46,6 +54,7 @@ public slots:
    void pushInstructionDown();
    void pushInstructionPrint();
    void pushInstructionPreview();
+   void changed(QMetaProperty, QVariant);
 
 private:
    void showChanges();
