@@ -1,6 +1,6 @@
 /*
 * BtSplashScreen.h is part of Brewtarget, and is Copyright Philip G. Lee
-* (rocketman768@gmail.com), 2009-2011.
+* (rocketman768@gmail.com), 2009-2013.
 *
 * Brewtarget is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -29,8 +29,9 @@ class BtSplashScreen;
 
 /*!
  * \class BtSplashScreen
- * A class for showing the brewtarget splash screen
- * on startup.
+ * \author Philip G. Lee
+ *
+ * \brief A class for showing the brewtarget splash screen on startup.
  */
 class BtSplashScreen : public QSplashScreen, public Ui::btSplashScreen
 {
@@ -38,7 +39,6 @@ class BtSplashScreen : public QSplashScreen, public Ui::btSplashScreen
 
 public:
    BtSplashScreen(QWidget* parent=0);
-   virtual ~BtSplashScreen();
    
    //! Overloaded from \b QSplashScreen
    void showMessage( const QString& message,
