@@ -1,6 +1,6 @@
 /*
  * FahrenheitTempUnitSystem.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,8 @@ FahrenheitTempUnitSystem::FahrenheitTempUnitSystem()
 {
 }
 
-QString FahrenheitTempUnitSystem::displayAmount( double amount, Unit* units )
+//! scale is ignored here, but must be included for the virtual method
+QString FahrenheitTempUnitSystem::displayAmount( double amount, Unit* units, unitScale scale )
 {
    QString SIUnitName = units->getSIUnitName();
    double SIAmount = units->toSI( amount );
@@ -35,9 +36,9 @@ QString FahrenheitTempUnitSystem::displayAmount( double amount, Unit* units )
    // Special cases. Make sure the unit isn't null and that we're
    // dealing with temperature.
    if( units == 0 || SIUnitName.compare("C") != 0 )
-      return QString("%1").arg(amount, fieldWidth, format, precision);
+      return QString("%L1").arg(amount, fieldWidth, format, precision);
 
-   ret = QString("%1 %2").arg(Units::fahrenheit->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::fahrenheit->getUnitName());
+   ret = QString("%L1 %2").arg(Units::fahrenheit->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::fahrenheit->getUnitName());
 
    return ret;
 }
@@ -62,3 +63,4 @@ void FahrenheitTempUnitSystem::ensureMapIsSetup()
    isMapSetup = true;
 }
 
+QString FahrenheitTempUnitSystem::unitType() { return "Fahrenheit"; }
