@@ -1,6 +1,6 @@
 /*
  * FermentableDialog.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,34 +21,41 @@
 #include <QString>
 #include <QInputDialog>
 #include <QList>
-#include <string>
+#include "FermentableEditor.h"
 #include "FermentableDialog.h"
 #include "FermentableTableModel.h"
-#include "observable.h"
+#include "FermentableSortFilterProxyModel.h"
 #include "database.h"
 #include "recipe.h"
 #include "MainWindow.h"
 #include "fermentable.h"
 
 FermentableDialog::FermentableDialog(MainWindow* parent)
-        : QDialog(parent)
+        : QDialog(parent), mainWindow(parent),
+          fermEdit(new FermentableEditor(this)), numFerms(0)
 {
    setupUi(this);
-   mainWindow = parent;
-   dbObs = 0;
-   numFerms = 0;
-   fermEdit = new FermentableEditor(this);
 
+   fermTableModel = new FermentableTableModel(fermentableTableWidget, false);
+   fermTableProxy = new FermentableSortFilterProxyModel(fermentableTableWidget);
+   fermTableProxy->setSourceModel(fermTableModel);
+   fermentableTableWidget->setModel(fermTableProxy);
+   fermentableTableWidget->setSortingEnabled(true);
+   fermentableTableWidget->sortByColumn( FERMNAMECOL, Qt::AscendingOrder );
+   fermTableProxy->setDynamicSortFilter(true);
+   
    connect( pushButton_addToRecipe, SIGNAL( clicked() ), this, SLOT( addFermentable() ) );
    connect( pushButton_edit, SIGNAL( clicked() ), this, SLOT( editSelected() ) );
    connect( pushButton_remove, SIGNAL( clicked() ), this, SLOT( removeFermentable() ) );
    connect( pushButton_new, SIGNAL( clicked() ), this, SLOT( newFermentable() ) );
    connect( fermentableTableWidget, SIGNAL( doubleClicked(const QModelIndex&) ), this, SLOT(addFermentable(const QModelIndex&)) );
+   
+   fermTableModel->observeDatabase(true);
 }
 
 void FermentableDialog::removeFermentable()
 {
-   QModelIndexList selected = fermentableTableWidget->selectedIndexes();
+   QModelIndexList selected = fermentableTableWidget->selectionModel()->selectedIndexes();
    QModelIndex translated;
    int row, size, i;
 
@@ -64,14 +71,14 @@ void FermentableDialog::removeFermentable()
          return;
    }
 
-   translated = fermentableTableWidget->getProxy()->mapToSource(selected[0]);
-   Fermentable* ferm = fermentableTableWidget->getModel()->getFermentable(translated.row());
-   dbObs->removeFermentable(ferm);
+   translated = fermTableProxy->mapToSource(selected[0]);
+   Fermentable* ferm = fermTableModel->getFermentable(translated.row());
+   Database::instance().removeFermentable(ferm);
 }
 
 void FermentableDialog::editSelected()
 {
-   QModelIndexList selected = fermentableTableWidget->selectedIndexes();
+   QModelIndexList selected = fermentableTableWidget->selectionModel()->selectedIndexes();
    QModelIndex translated;
    int row, size, i;
 
@@ -87,41 +94,12 @@ void FermentableDialog::editSelected()
          return;
    }
 
-   translated = fermentableTableWidget->getProxy()->mapToSource(selected[0]);
-   Fermentable* ferm = fermentableTableWidget->getModel()->getFermentable(translated.row());
+   translated = fermTableProxy->mapToSource(selected[0]);
+   Fermentable* ferm = fermTableModel->getFermentable(translated.row());
    fermEdit->setFermentable(ferm);
    fermEdit->show();
 }
 
-void FermentableDialog::notify(Observable *notifier, QVariant info)
-{
-   if( notifier != dbObs || (info.toInt() != DBFERM && info.toInt() != DBALL) )
-      return;
-
-   fermentableTableWidget->getModel()->removeAll();
-   populateTable();
-}
-
-void FermentableDialog::startObservingDB()
-{
-   dbObs = Database::getDatabase();
-   setObserved(dbObs);
-   populateTable();
-}
-
-void FermentableDialog::populateTable()
-{
-   QList<Fermentable*>::iterator it, end;
-
-   if( ! Database::isInitialized() )
-      return;
-
-   numFerms = dbObs->getNumFermentables();
-   end = dbObs->getFermentableEnd();
-   for( it = dbObs->getFermentableBegin(); it != end; ++it )
-      fermentableTableWidget->getModel()->addFermentable(*it);
-}
-
 void FermentableDialog::addFermentable(const QModelIndex& index)
 {
    QModelIndex translated;
@@ -129,7 +107,7 @@ void FermentableDialog::addFermentable(const QModelIndex& index)
    // If there is no provided index, get the selected index.
    if( !index.isValid() )
    {
-      QModelIndexList selected = fermentableTableWidget->selectedIndexes();
+      QModelIndexList selected = fermentableTableWidget->selectionModel()->selectedIndexes();
       int row, size, i;
 
       size = selected.size();
@@ -144,7 +122,7 @@ void FermentableDialog::addFermentable(const QModelIndex& index)
             return;
       }
       
-      translated = fermentableTableWidget->getProxy()->mapToSource(selected[0]);
+      translated = fermTableProxy->mapToSource(selected[0]);
    }
    else
    {
@@ -152,13 +130,14 @@ void FermentableDialog::addFermentable(const QModelIndex& index)
       // this keeps us from adding something to the recipe when we just want to edit
       // one of the other fermentable fields.
       if( index.column() == FERMNAMECOL )
-         translated = fermentableTableWidget->getProxy()->mapToSource(index);
+         translated = fermTableProxy->mapToSource(index);
       else
          return;
    }
    
-   Fermentable *ferm = fermentableTableWidget->getModel()->getFermentable(translated.row());
-   mainWindow->addFermentableToRecipe(new Fermentable(*ferm) ); // Need to add a copy so we don't change the database.
+   Fermentable *ferm = fermTableModel->getFermentable(translated.row());
+   
+   Database::instance().addToRecipe( mainWindow->currentRecipe(), ferm );
 }
 
 void FermentableDialog::newFermentable()
@@ -168,11 +147,8 @@ void FermentableDialog::newFermentable()
    if( name.isEmpty() )
       return;
    
-   Fermentable *ferm = new Fermentable();
-   QString stdname = name;
-   ferm->setName(stdname);
-
-   dbObs->addFermentable(ferm);
+   Fermentable* ferm = Database::instance().newFermentable();
+   ferm->setName(name);
    fermEdit->setFermentable(ferm);
    fermEdit->show();
 }
