@@ -1,6 +1,6 @@
 /*
  * FermentableEditor.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +21,21 @@
 
 class FermentableEditor;
 
+#include <QDialog>
+#include <QMetaProperty>
+#include <QVariant>
 #include "ui_fermentableEditor.h"
-#include "fermentable.h"
-#include "observable.h"
 
-class FermentableEditor : public QDialog, private Ui::fermentableEditor, Observer
+// Forward declarations.
+class Fermentable;
+
+/*!
+ * \class FermentableEditor
+ * \author Philip G. Lee
+ *
+ * \brief Fermentable view/controller dialog that allows you to edit Fermentables.
+ */
+class FermentableEditor : public QDialog, private Ui::fermentableEditor
 {
    Q_OBJECT
 
@@ -37,13 +47,15 @@ public:
 public slots:
    void save();
    void clearAndClose();
+   void changed(QMetaProperty,QVariant);
 
 private:
    Fermentable* obsFerm;
-
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer
-   void showChanges();
+   /*! Updates the UI elements effected by the \b metaProp of
+    *  the fermentable we are watching. If \b metaProp is null,
+    *  then update all the UI elements at once.
+    */
+   void showChanges(QMetaProperty* metaProp = 0);
 };
 
 #endif   /* _FERMENTABLEEDITOR_H */
-
