@@ -1,6 +1,6 @@
 /*
  * FermentableSortFilterProxyModel.cpp is part of Brewtarget, and is Copyright Mik
- * Firestone (mikfire@gmail.com), 2010-2011.
+ * Firestone (mikfire@gmail.com), 2010-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,12 +19,15 @@
 #include "unit.h"
 #include "FermentableSortFilterProxyModel.h"
 #include "FermentableTableModel.h"
+#include "fermentable.h"
+#include "brewtarget.h"
 #include <iostream>
 #include <QDebug>
 
-FermentableSortFilterProxyModel::FermentableSortFilterProxyModel(QObject *parent) 
+FermentableSortFilterProxyModel::FermentableSortFilterProxyModel(QObject *parent, bool filt) 
 : QSortFilterProxyModel(parent)
 {
+   filter = filt;
 }
 
 bool FermentableSortFilterProxyModel::lessThan(const QModelIndex &left, 
@@ -38,10 +41,10 @@ bool FermentableSortFilterProxyModel::lessThan(const QModelIndex &left,
       case FERMAMOUNTCOL:
          // This is a bit twisted. If the numbers are equal, reset the left
          // and right to the names and let it hit the default
-         if (Unit::qstringToSI(leftFermentable.toString()) == Unit::qstringToSI(rightFermentable.toString()))
+         if (Brewtarget::weightQStringToSI(leftFermentable.toString()) == Brewtarget::weightQStringToSI(rightFermentable.toString()))
             return getName(right) < getName(left);
          else
-            return Unit::qstringToSI(leftFermentable.toString()) < Unit::qstringToSI(rightFermentable.toString());
+            return Brewtarget::weightQStringToSI(leftFermentable.toString()) < Brewtarget::weightQStringToSI(rightFermentable.toString());
       case FERMYIELDCOL:
          if (leftFermentable.toDouble() == rightFermentable.toDouble() )
             return getName(right) < getName(left);
@@ -62,3 +65,9 @@ QString FermentableSortFilterProxyModel::getName( const QModelIndex &index ) con
    QVariant info = sourceModel()->data(QAbstractItemModel::createIndex(index.row(),FERMNAMECOL));
    return info.toString();
 }
+
+bool FermentableSortFilterProxyModel::filterAcceptsRow( int source_row, const QModelIndex &source_parent) const
+{
+   FermentableTableModel* model = qobject_cast<FermentableTableModel*>(sourceModel());
+   return ! filter || model->getFermentable(source_row)->display();
+}
