@@ -1,6 +1,7 @@
 /*
- * FermentableSortFilterProxyModel.cpp is part of Brewtarget, and is Copyright Mik
- * Firestone (mikfire@gmail.com), 2010-2011.
+ * FermentableSortFilterProxyModel.h is part of Brewtarget, and is Copyright Mik
+ * Firestone (mikfire@gmail.com) and Philip G. Lee (rocketman768@gmail.com,
+ * 2010-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,17 +24,27 @@ class FermentableSortFilterProxyModel;
 
 #include <QSortFilterProxyModel>
 
+/*!
+ * \class FermentableSortFilterProxyModel
+ * \author Mik Firestone
+ * \author Philip G. Lee
+ *
+ * \brief Proxy model for sorting Fermentables.
+ */
 class FermentableSortFilterProxyModel : public QSortFilterProxyModel
 {
    Q_OBJECT
 
 public:
-   FermentableSortFilterProxyModel(QObject *parent = 0);
+   FermentableSortFilterProxyModel(QObject *parent = 0, bool filt = true);
 
 protected:
    bool lessThan(const QModelIndex &left, const QModelIndex &right) const;
+   bool filterAcceptsRow( int source_row, const QModelIndex &source_parent) const;
 
 private:
+   bool filter;
+
    QString getName( const QModelIndex &index ) const;
 };
 
