@@ -1,6 +1,6 @@
 /*
  * FermentableTableModel.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,68 +23,116 @@ class FermentableTableModel;
 class FermentableItemDelegate;
 
 #include <QAbstractTableModel>
+#include <QTableView>
 #include <QWidget>
 #include <QModelIndex>
 #include <QVariant>
-#include <Qt>
-#include <QStringList>
-#include <QStyledItemDelegate>
+#include <QMetaProperty>
+#include <QItemDelegate>
 #include <QAbstractItemDelegate>
-#include <QVector>
-#include "fermentable.h"
-#include "FermentableTableWidget.h"
-#include "observable.h"
+#include <QList>
+#include "unit.h"
+
+// Forward declarations.
+class Fermentable;
+class Recipe;
 
 enum{FERMNAMECOL, FERMTYPECOL, FERMAMOUNTCOL, FERMISMASHEDCOL, FERMAFTERBOIL, FERMYIELDCOL, FERMCOLORCOL, FERMNUMCOLS /*This one MUST be last*/};
 
-class FermentableTableModel : public QAbstractTableModel, public MultipleObserver
+/*!
+ * \class FermentableTableModel
+ * \author Philip G. Lee
+ *
+ * \brief A table model for a list of fermentables.
+ */
+class FermentableTableModel : public QAbstractTableModel
 {
    Q_OBJECT
-           
+
 public:
-   FermentableTableModel(FermentableTableWidget* parent=0);
+   FermentableTableModel(QTableView* parent=0, bool editable=true);
    virtual ~FermentableTableModel() {}
-   void addFermentable(Fermentable* ferm);
-   bool removeFermentable(Fermentable* ferm); // Returns true if "ferm" is successfully found and removed.
+   //! \brief Observe a recipe's list of fermentables.
+   void observeRecipe(Recipe* rec);
+   //! \brief If true, we model the database's list of fermentables.
+   void observeDatabase(bool val);
+   //! \brief Watch all the \b ferms for changes.
+   void addFermentables(QList<Fermentable*> ferms);
+   //! \brief Clear the model.
    void removeAll();
+   //! \brief Return the \c i-th fermentable in the model.
    Fermentable* getFermentable(unsigned int i);
+   //! \brief True if you want to display percent of each grain in the row header.
    void setDisplayPercentages( bool var );
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer via MultipleObserver.
-   
-   // Inherit the following from QAbstractItemModel via QAbstractTableModel
+
+   unitDisplay displayUnit(int column) const;
+   unitScale displayScale(int column) const;
+   void setDisplayUnit(int column, unitDisplay displayUnit);
+   void setDisplayScale(int column, unitScale displayScale);
+
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual QVariant data( const QModelIndex& index, int role = Qt::DisplayRole ) const;
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual QVariant headerData( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual Qt::ItemFlags flags(const QModelIndex& index ) const;
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual bool setData( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
    
-   FermentableTableWidget* parentTableWidget;
+   QTableView* parentTableWidget;
    
+public slots:
+   //! \brief Watch \b ferm for changes.
+   void addFermentable(Fermentable* ferm);
+   //! \returns true if "ferm" is successfully found and removed.
+   bool removeFermentable(Fermentable* ferm);
+
+private slots:
+   //! \brief Catch changes to Recipe, Database, and Fermentable.
+   void changed(QMetaProperty, QVariant);
+
 private:
+   //! \brief Recalculate the total amount of grains in the model.
    void updateTotalGrains();
+   QString generateName(int column) const;
    
-   QVector<Fermentable*> fermObs;
-   bool displayPercentages; // True if you want to display percent of each grain in the row header.
+   bool editable;
+   QList<Fermentable*> fermObs;
+   Recipe* recObs;
+   bool displayPercentages;
    double totalFermMass_kg;
+   
 };
 
-class FermentableItemDelegate : public QStyledItemDelegate
+/*!
+ * \brief An item delegate for Fermentable tables.
+ * \sa FermentableTableModel.
+ * 
+ * \author Philip G. Lee
+ */
+class FermentableItemDelegate : public QItemDelegate
 {
    Q_OBJECT
            
 public:
    FermentableItemDelegate(QObject* parent = 0);
    
-   // Inherited functions.
+   //! \brief Reimplemented from QItemDelegate.
    virtual QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
+   //! \brief Reimplemented from QItemDelegate.
    virtual void setEditorData(QWidget *editor, const QModelIndex &index) const;
+   //! \brief Reimplemented from QItemDelegate.
    virtual void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
+   //! \brief Reimplemented from QItemDelegate.
    virtual void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    //virtual void paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const;
    
-public slots:
-   void destroyWidget(QWidget* widget, QAbstractItemDelegate::EndEditHint hint);
+//public slots:
+//   void destroyWidget(QWidget* widget, QAbstractItemDelegate::EndEditHint hint);
 private:
 };
 
