@@ -1,6 +1,6 @@
 /*
  * HeatCalculations.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,12 @@
 
 class HeatCalculations;
 
+/*!
+ * \class
+ * \author Philip G. Lee
+ *
+ * \brief Algorithms and constants related to the thermodynamics of beer.
+ */
 class HeatCalculations
 {
 public:
@@ -35,13 +41,6 @@ public:
    static double Cw_JKgK;
    static double Cw_calGC;
    static double Cgrain_calGC;
-
-   /***Density***/
-   static double rhoGrain_KgL;
-   
-   /***Absorption***/
-   static double absorption_LKg;
 };
 
 #endif   /* _HEATCALCULATIONS_H */
-
