@@ -1,6 +1,6 @@
 /*
  * HopDialog.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,37 +24,46 @@ class HopDialog;
 #include <QWidget>
 #include <QDialog>
 #include <QVariant>
+#include <QMetaProperty>
 #include "ui_hopDialog.h"
-#include "observable.h"
 #include "database.h"
-class MainWindow;
-#include "HopEditor.h"
 
-class HopDialog : public QDialog, public Ui::hopDialog, public Observer
+// Forward declarations.
+class MainWindow;
+class HopEditor;
+class HopTableModel;
+class HopSortFilterProxyModel;
+
+/*!
+ * \class HopDialog
+ * \author Philip G. Lee
+ *
+ * \brief View/controller class for showing/editing the list of hops in the database.
+ */
+class HopDialog : public QDialog, public Ui::hopDialog
 {
    Q_OBJECT
 
 public:
    HopDialog(MainWindow* parent);
    virtual ~HopDialog() {}
-   void startObservingDB();
-   virtual void notify(Observable *notifier, QVariant info = QVariant()); // From Observer
 
 public slots:
+   //! Add selected hop to current recipe.
    void addHop(const QModelIndex& = QModelIndex());
+   //! Delete the selected hop from the database.
    void removeHop();
+   //! Bring up the editor for the selected hop.
    void editSelected();
+   //! Create a new hop.
    void newHop();
 
 private:
-   Database* dbObs;
    MainWindow* mainWindow;
    HopEditor* hopEditor;
-   unsigned int numHops;
-
-   void populateTable();
+   HopTableModel* hopTableModel;
+   HopSortFilterProxyModel* hopTableProxy;
+   int numHops;
 };
 
-
 #endif   /* _HOPDIALOG_H */
-
