@@ -1,6 +1,6 @@
 /*
  * HopEditor.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,28 +22,42 @@
 class HopEditor;
 
 #include "ui_hopEditor.h"
-#include "hop.h"
-#include "observable.h"
+#include <QMetaProperty>
+#include <QVariant>
 
-class HopEditor : public QDialog, private Ui::hopEditor, Observer
+// Forward declarations.
+class Hop;
+
+/*!
+ * \class HopEditor
+ * \author Philip G. Lee
+ *
+ * \brief View/controller class for modifying hops.
+ */
+class HopEditor : public QDialog, private Ui::hopEditor
 {
    Q_OBJECT
 
 public:
    HopEditor( QWidget *parent=0 );
    virtual ~HopEditor() {}
+   //! Edit the given hop.
    void setHop( Hop* h );
 
 public slots:
+   //! Save the changes.
    void save();
+   //! Clear the dialog and close it.
    void clearAndClose();
+   void changed(QMetaProperty,QVariant);
 
 private:
    Hop* obsHop;
 
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer
-   void showChanges();
+   /*! Updates the UI elements based on \b prop.
+    *  If null, updates all UI elements.
+    */
+   void showChanges(QMetaProperty* prop = 0);
 };
 
 #endif   /* _HOPEDITOR_H */
-
