@@ -1,6 +1,7 @@
 /*
  * HopSortFilterProxyModel.cpp is part of Brewtarget, and is Copyright Mik
- * Firestone (mikfire@gmail.com), 2010-2011.
+ * Firestone (mikfire@gmail.com) and Philip G. Lee (rocketman768@gmail.com),
+ * 2010-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,16 +24,26 @@ class HopSortFilterProxyModel;
 
 #include <QSortFilterProxyModel>
 
+/*!
+ * \class HopSortFilterProxyModel
+ * \author Mik Firestone
+ * \author Philip G. Lee
+ *
+ * \brief Proxy model for sorting hops.
+ */
 class HopSortFilterProxyModel : public QSortFilterProxyModel
 {
    Q_OBJECT
 
 public:
-   HopSortFilterProxyModel(QObject *parent = 0);
+   HopSortFilterProxyModel(QObject *parent = 0, bool filt = true);
 
 protected:
    bool lessThan(const QModelIndex &left, const QModelIndex &right) const;
+   bool filterAcceptsRow( int source_row, const QModelIndex &source_parent) const;
 
+private:
+   bool filter;
 };
 
 #endif
