@@ -1,6 +1,6 @@
 /*
  * HopTableModel.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,29 +21,35 @@
 #include <QWidget>
 #include <QModelIndex>
 #include <QVariant>
-#include <Qt>
 #include <QItemDelegate>
 #include <QStyleOptionViewItem>
 #include <QComboBox>
 #include <QLineEdit>
 
+#include "database.h"
 #include "hop.h"
 #include <QString>
 #include <QVector>
-#include <iostream>
 #include "hop.h"
-#include "observable.h"
 #include "HopTableModel.h"
 #include "unit.h"
 #include "brewtarget.h"
 
-HopTableModel::HopTableModel(HopTableWidget* parent)
-: QAbstractTableModel(parent), MultipleObserver()
+HopTableModel::HopTableModel(QTableView* parent, bool editable)
+   : QAbstractTableModel(parent), colFlags(HOPNUMCOLS), recObs(0), parentTableWidget(parent), showIBUs(false)
 {
    hopObs.clear();
-   parentTableWidget = parent;
-   showIBUs = false;
-   recObs = 0;
+   setObjectName("hopTable");
+   
+   int i;
+   for( i = 0; i < HOPNUMCOLS; ++i )
+   {
+      if( i == HOPNAMECOL )
+         colFlags[i] = Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled;
+      else
+         colFlags[i] = Qt::ItemIsSelectable | (editable ? Qt::ItemIsEditable : Qt::NoItemFlags) | Qt::ItemIsDragEnabled |
+            Qt::ItemIsEnabled;
+   }
 }
 
 HopTableModel::~HopTableModel()
@@ -51,19 +57,62 @@ HopTableModel::~HopTableModel()
    hopObs.clear();
 }
 
+void HopTableModel::observeRecipe(Recipe* rec)
+{
+   if( recObs )
+   {
+      disconnect( recObs, 0, this, 0 );
+      removeAll();
+   }
+   
+   recObs = rec;
+   if( recObs )
+   {
+      connect( recObs, SIGNAL(changed(QMetaProperty,QVariant)), this, SLOT(changed(QMetaProperty,QVariant)) );
+      addHops( recObs->hops() );
+   }
+}
+
+void HopTableModel::observeDatabase(bool val)
+{
+   if( val )
+   {
+      observeRecipe(0);
+      removeAll();
+      connect( &(Database::instance()), SIGNAL(newHopSignal(Hop*)), this, SLOT(addHop(Hop*)) );
+      connect( &(Database::instance()), SIGNAL(deletedHopSignal(Hop*)), this, SLOT(removeHop(Hop*)) );
+      addHops( Database::instance().hops() );
+   }
+   else
+   {
+      removeAll();
+      disconnect( &(Database::instance()), 0, this, 0 );
+   }
+}
+
 void HopTableModel::addHop(Hop* hop)
 {
-   QVector<Hop*>::iterator iter;
-   
-   //Check to see if it's already in the list
-   for( iter=hopObs.begin(); iter != hopObs.end(); iter++ )
-      if( *iter == hop )
-         return;
-   
-   hopObs.push_back(hop);
-   addObserved(hop);
-   reset(); // Tell everybody that the table has changed.
+   if( hop == 0 || hopObs.contains(hop) )
+      return;
 
+   // If we are observing the database, ensure that the item is undeleted and
+   // fit to display.
+   if(
+      recObs == 0 &&
+      (
+         hop->deleted() ||
+         !hop->display()
+      )
+   )
+      return;
+   
+   int size = hopObs.size();
+   beginInsertRows( QModelIndex(), size, size );
+   hopObs.append(hop);
+   connect( hop, SIGNAL(changed(QMetaProperty,QVariant)), this, SLOT(changed(QMetaProperty,QVariant)) );
+   //reset(); // Tell everybody that the table has changed.
+   endInsertRows();
+   
    if( parentTableWidget )
    {
       parentTableWidget->resizeColumnsToContents();
@@ -71,70 +120,109 @@ void HopTableModel::addHop(Hop* hop)
    }
 }
 
+void HopTableModel::addHops(QList<Hop*> hops)
+{
+   QList<Hop*>::iterator i;
+   QList<Hop*> tmp;
+   
+   for( i = hops.begin(); i != hops.end(); i++ )
+   {
+      if( !hopObs.contains(*i) )
+         tmp.append(*i);
+   }
+   
+   int size = hopObs.size();
+   if (size+tmp.size())
+   {
+      beginInsertRows( QModelIndex(), size, size+tmp.size()-1 );
+      hopObs.append(tmp);
+      
+      for( i = tmp.begin(); i != tmp.end(); i++ )
+         connect( *i, SIGNAL(changed(QMetaProperty,QVariant)), this, SLOT(changed(QMetaProperty,QVariant)) );
+
+      endInsertRows();
+   }
+
+   if( parentTableWidget )
+   {
+      parentTableWidget->resizeColumnsToContents();
+      parentTableWidget->resizeRowsToContents();
+   }
+   
+}
+
+bool HopTableModel::removeHop(Hop* hop)
+{
+   int i;
+   i = hopObs.indexOf(hop);
+   if( i >= 0 )
+   {
+      beginRemoveRows( QModelIndex(), i, i );
+      disconnect( hop, 0, this, 0 );
+      hopObs.removeAt(i);
+      //reset(); // Tell everybody the table has changed.
+      endRemoveRows();
+
+      if(parentTableWidget)
+      {
+         parentTableWidget->resizeColumnsToContents();
+         parentTableWidget->resizeRowsToContents();
+      }
+
+      return true;
+   }
+      
+   return false;
+}
+
 void HopTableModel::setShowIBUs( bool var )
 {
    showIBUs = var;
 }
 
-void HopTableModel::setRecipe( Recipe* rec )
+void HopTableModel::removeAll()
 {
-   if( recObs )
-      removeObserved(recObs);
-   
-   if( rec )
+   if (hopObs.size())
    {
-      addObserved(rec);
-      recObs = rec;
+      beginRemoveRows( QModelIndex(), 0, hopObs.size()-1 );
+      while( !hopObs.isEmpty() )
+      {
+         disconnect( hopObs.takeLast(), 0, this, 0 );
+      }
+      endRemoveRows();
    }
 }
 
-bool HopTableModel::removeHop(Hop* hop)
-{
-   QVector<Hop*>::iterator iter;
-   
-   for( iter=hopObs.begin(); iter != hopObs.end(); iter++ )
-      if( *iter == hop )
-      {
-         hopObs.erase(iter);
-         removeObserved(hop);
-         reset(); // Tell everybody the table has changed.
-         
-         if( parentTableWidget )
-         {
-            parentTableWidget->resizeColumnsToContents();
-            parentTableWidget->resizeRowsToContents();
-         }
-         
-         return true;
-      }
-   
-   return false;
-}
-
-void HopTableModel::removeAll()
+void HopTableModel::changed(QMetaProperty prop, QVariant /*val*/)
 {
    int i;
-
-   for( i = 0; i < hopObs.size(); ++i )
-      removeObserved(hopObs[i]);
-
-   hopObs.clear();
-   reset();
-}
-
-void HopTableModel::notify(Observable* notifier, QVariant info)
-{
-   int i;
-   
-   if( notifier == recObs )
-      emit headerDataChanged( Qt::Vertical, 0, rowCount()-1 );
    
    // Find the notifier in the list
-   for( i = 0; i < (int)hopObs.size(); ++i )
+   Hop* hopSender = qobject_cast<Hop*>(sender());
+   if( hopSender )
    {
-      if( notifier == hopObs[i] )
-         emit dataChanged( QAbstractItemModel::createIndex(i, 0),
-                           QAbstractItemModel::createIndex(i, HOPNUMCOLS));
+      i = hopObs.indexOf(hopSender);
+      if( i < 0 )
+         return;
+      
+      emit dataChanged( QAbstractItemModel::createIndex(i, 0),
+                        QAbstractItemModel::createIndex(i, HOPNUMCOLS-1));
+      emit headerDataChanged( Qt::Vertical, i, i );
+      return;
+   }
+   
+   // See if sender is our recipe.
+   Recipe* recSender = qobject_cast<Recipe*>(sender());
+   if( recSender && recSender == recObs )
+   {
+      if( QString(prop.name()) == "hops" )
+      {
+         removeAll();
+         addHops( recObs->hops() );
+      }
+      if( rowCount() > 0 )
+         emit headerDataChanged( Qt::Vertical, 0, rowCount()-1 );
+      return;
    }
 }
 
@@ -151,11 +239,14 @@ int HopTableModel::columnCount(const QModelIndex& /*parent*/) const
 QVariant HopTableModel::data( const QModelIndex& index, int role ) const
 {
    Hop* row;
+   int col = index.column();
+   unitScale scale;
+   unitDisplay unit;
    
    // Ensure the row is ok.
    if( index.row() >= (int)hopObs.size() )
    {
-      Brewtarget::log(Brewtarget::WARNING, tr("Bad model index. row = %1").arg(index.row()));
+      Brewtarget::log(Brewtarget::WARNING, QString("Bad model index. row = %1").arg(index.row()));
       return QVariant();
    }
    else
@@ -165,40 +256,43 @@ QVariant HopTableModel::data( const QModelIndex& index, int role ) const
    {
       case HOPNAMECOL:
          if( role == Qt::DisplayRole )
-            return QVariant(row->getName());
+            return QVariant(row->name());
          else
             return QVariant();
       case HOPALPHACOL:
          if( role == Qt::DisplayRole )
-            return QVariant( Brewtarget::displayAmount(row->getAlpha_pct(), 0) );
+            return QVariant( Brewtarget::displayAmount(row->alpha_pct(), 0) );
          else
             return QVariant();
       case HOPAMOUNTCOL:
-         if( role == Qt::DisplayRole )
-            return QVariant( Brewtarget::displayAmount(row->getAmount_kg(), Units::kilograms) );
-         else
+         if( role != Qt::DisplayRole )
             return QVariant();
+         unit = displayUnit(col);
+         scale = displayScale(col);
+
+         return QVariant(Brewtarget::displayAmount(row->amount_kg(), Units::kilograms, 3, unit, scale));
+
       case HOPUSECOL:
          if( role == Qt::DisplayRole )
-            return QVariant(row->getUseStringTr());
+            return QVariant(row->useStringTr());
          else if( role == Qt::UserRole )
-            return QVariant(row->getUse());
+            return QVariant(row->use());
          else
             return QVariant();
       case HOPTIMECOL:
          if( role == Qt::DisplayRole )
-            return QVariant( Brewtarget::displayAmount(row->getTime_min(), Units::minutes) );
+            return QVariant( Brewtarget::displayAmount(row->time_min(), Units::minutes) );
          else
             return QVariant();
       case HOPFORMCOL:
         if ( role == Qt::DisplayRole )
-          return QVariant( row->getFormStringTr() );
+          return QVariant( row->formStringTr() );
         else if ( role == Qt::UserRole )
-           return QVariant( row->getForm());
+           return QVariant( row->form());
         else
            return QVariant();
       default :
-         Brewtarget::log(Brewtarget::WARNING, tr("HopTableModel::data Bad column: %1").arg(index.column()));
+         Brewtarget::log(Brewtarget::WARNING, QString("HopTableModel::data Bad column: %1").arg(index.column()));
          return QVariant();
    }
 }
@@ -220,30 +314,27 @@ QVariant HopTableModel::headerData( int section, Qt::Orientation orientation, in
          case HOPTIMECOL:
             return QVariant(tr("Time"));
          case HOPFORMCOL:
-           return QVariant(tr("Form"));
+            return QVariant(tr("Form"));
          default:
-            Brewtarget::log(Brewtarget::WARNING, tr("HopTableModel::headerdata Bad column: %1").arg(section));
+            Brewtarget::log(Brewtarget::WARNING, QString("HopTableModel::headerdata Bad column: %1").arg(section));
             return QVariant();
       }
    }
    else if( showIBUs && recObs && orientation == Qt::Vertical && role == Qt::DisplayRole )
    {
-      double ibus = recObs->getIBUFromHop( section );
-      return QVariant( QString("%1 IBU").arg( ibus, 0, 'f', 1 ) );
+      QList<double> ibus = recObs->IBUs();
+
+      if ( ibus.size() > section )
+         return QVariant( QString("%L1 IBU").arg( ibus.at(section), 0, 'f', 1 ) );
    }
-   else
-      return QVariant();
+   return QVariant();
 }
 
 Qt::ItemFlags HopTableModel::flags(const QModelIndex& index ) const
 {
    int col = index.column();
    
-   if( col == HOPNAMECOL )
-      return Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled;
-   else
-      return Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsDragEnabled |
-         Qt::ItemIsEnabled;
+   return colFlags[col];
 }
 
 bool HopTableModel::setData( const QModelIndex& index, const QVariant& value, int role )
@@ -307,7 +398,8 @@ bool HopTableModel::setData( const QModelIndex& index, const QVariant& value, in
       case HOPTIMECOL:
          if( value.canConvert(QVariant::String) )
          {
-            row->setTime_min( Brewtarget::timeQStringToSI(value.toString()) );
+            double min = Brewtarget::timeQStringToSI(value.toString());
+            row->setTime_min( min );
             headerDataChanged( Qt::Vertical, index.row(), index.row() ); // Need to re-show header (IBUs).
             return true;
          }
@@ -319,17 +411,96 @@ bool HopTableModel::setData( const QModelIndex& index, const QVariant& value, in
    }
 }
 
-// Returns null on failure.
-Hop* HopTableModel::getHop(unsigned int i)
+unitDisplay HopTableModel::displayUnit(int column) const
+{ 
+   QString attribute = generateName(column);
+
+   if ( attribute.isEmpty() )
+      return noUnit;
+
+   return (unitDisplay)Brewtarget::option(attribute, QVariant(-1), this, Brewtarget::UNIT).toInt();
+}
+
+unitScale HopTableModel::displayScale(int column) const
+{ 
+   QString attribute = generateName(column);
+
+   if ( attribute.isEmpty() )
+      return noScale;
+
+   return (unitScale)Brewtarget::option(attribute, QVariant(-1), this, Brewtarget::SCALE).toInt();
+}
+
+// We need to:
+//   o clear the custom scale if set
+//   o clear any custom unit from the rows
+//      o which should have the side effect of clearing any scale
+void HopTableModel::setDisplayUnit(int column, unitDisplay displayUnit) 
 {
-   //std::cerr << "HopTableModel::getHop( " << i << "/" << hopObs.size()  << " )" << std::endl;
-   if( static_cast<int>(i) < hopObs.size() )
-      return hopObs[i];
-   else
+   // Hop* row; // disabled per-cell magic
+   QString attribute = generateName(column);
+
+   if ( attribute.isEmpty() )
+      return;
+
+   Brewtarget::setOption(attribute,displayUnit,this,Brewtarget::UNIT); 
+   Brewtarget::setOption(attribute,noScale,this,Brewtarget::SCALE);
+
+   /* Disabled cell-specific code
+   for (int i = 0; i < rowCount(); ++i )
    {
-      Brewtarget::logW( QString("HopTableModel::getHop( %1/%2 )").arg(i).arg(hopObs.size()) );
-      return 0;
+      row = getHop(i);
+      row->setDisplayUnit(noUnit);
    }
+   */
+}
+
+// Setting the scale should clear any cell-level scaling options
+void HopTableModel::setDisplayScale(int column, unitScale displayScale) 
+{ 
+   // Fermentable* row; //disabled per-cell magic
+
+   QString attribute = generateName(column);
+
+   if ( attribute.isEmpty() )
+      return;
+
+   Brewtarget::setOption(attribute,displayScale,this,Brewtarget::SCALE); 
+
+   /* disabled cell-specific code
+   for (int i = 0; i < rowCount(); ++i )
+   {
+      row = getHop(i);
+      row->setDisplayScale(noScale);
+   }
+   */
+}
+
+QString HopTableModel::generateName(int column) const
+{
+   QString attribute;
+
+   switch(column)
+   {
+      case HOPAMOUNTCOL:
+         attribute = "amount_kg";
+         break;
+      default:
+         attribute = "";
+   }
+   return attribute;
+}
+
+// Returns null on failure.
+Hop* HopTableModel::getHop(unsigned int i){
+    //std::cerr << "HopTableModel::getHop( " << i << "/" << hopObs.size()  << " )" << std::endl;
+    if(!(hopObs.isEmpty())){
+        if(static_cast<int>(i) < hopObs.size())
+            return hopObs[i];
+    }
+    else
+        Brewtarget::logW( QString("HopTableModel::getHop( %1/%2 )").arg(i).arg(hopObs.size()) );
+    return 0;
 }
 
 //==========================CLASS HopItemDelegate===============================
@@ -345,11 +516,12 @@ QWidget* HopItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewIt
    {
       QComboBox *box = new QComboBox(parent);
 
-      box->addItem(tr("Boil"));
-      box->addItem(tr("Dry Hop"));
+      // NOTE: these need to be in the same order as the Hop::Use enum.
       box->addItem(tr("Mash"));
       box->addItem(tr("First Wort"));
+      box->addItem(tr("Boil"));
       box->addItem(tr("Aroma"));
+      box->addItem(tr("Dry Hop"));
       box->setSizeAdjustPolicy(QComboBox::AdjustToContents);
 
       return box;
@@ -400,20 +572,27 @@ void HopItemDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, c
    if ( index.column() == HOPUSECOL )
    {
       QComboBox* box = (QComboBox*)editor;
-       int value = box->currentIndex();
-       model->setData(index, value, Qt::EditRole);
+      int value = box->currentIndex();
+      int ndx = model->data(index, Qt::UserRole).toInt();
+
+      if ( value != ndx )
+         model->setData(index, value, Qt::EditRole);
    }
    else if (index.column() == HOPFORMCOL )
    {
-       QComboBox* box = (QComboBox*)editor;
-       int value = box->currentIndex();
-       model->setData(index, value, Qt::EditRole);
+      QComboBox* box = (QComboBox*)editor;
+      int value = box->currentIndex();
+      int ndx = model->data(index, Qt::UserRole).toInt();
+
+      if ( value != ndx )
+         model->setData(index, value, Qt::EditRole);
    }
    else
    {
-       QLineEdit* line = (QLineEdit*)editor;
-      model->setData(index, line->text(), Qt::EditRole);
-    }
+      QLineEdit* line = (QLineEdit*)editor;
+      if ( line->isModified() )
+         model->setData(index, line->text(), Qt::EditRole);
+   }
 }
 
 void HopItemDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex& /*index*/) const
