@@ -1,6 +1,6 @@
 /*
  * HopTableModel.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,46 +26,86 @@ class HopItemDelegate;
 #include <QWidget>
 #include <QModelIndex>
 #include <QVariant>
-#include <Qt>
+#include <QTableView>
 #include <QItemDelegate>
 #include <QVector>
 #include "hop.h"
-#include "observable.h"
-#include "HopTableWidget.h"
 #include "recipe.h"
 
 enum{HOPNAMECOL, HOPALPHACOL, HOPAMOUNTCOL, HOPFORMCOL, HOPUSECOL, HOPTIMECOL, HOPNUMCOLS /*This one MUST be last*/};
 
-class HopTableModel : public QAbstractTableModel, public MultipleObserver
+/*!
+ * \class HopTableModel
+ * \author Philip G. Lee
+ *
+ * \brief Model class for a list of hops.
+ */
+class HopTableModel : public QAbstractTableModel
 {
    Q_OBJECT
            
 public:
-   HopTableModel(HopTableWidget* parent=0);
+
+   HopTableModel(QTableView* parent=0, bool editable=true);
    virtual ~HopTableModel();
-   void setRecipe( Recipe* rec ); // You need to set a recipe if you want to show IBUs.
-   void setShowIBUs( bool var ); // If you want to show IBUs.
-   void addHop(Hop* hop);
+   //! \brief Observe a recipe's list of fermentables.
+   void observeRecipe(Recipe* rec);
+   //! \brief If true, we model the database's list of hops.
+   void observeDatabase(bool val);
+   //! \brief Show ibus in the vertical header.
+   void setShowIBUs( bool var );
+   //! \brief Watch all the \c hops for changes.
+   void addHops(QList<Hop*> hops);
+   //! \brief Return the \c i-th hop in the model.
    Hop* getHop(unsigned int i);
-   bool removeHop(Hop* hop); // Returns true if "hop" is successfully found and removed.
+   //! \brief Clear the model.
    void removeAll();
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer via MultipleObserver.
    
-   // Inherit the following from QAbstractItemModel via QAbstractTableModel
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual QVariant data( const QModelIndex& index, int role = Qt::DisplayRole ) const;
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual QVariant headerData( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual Qt::ItemFlags flags(const QModelIndex& index ) const;
+   //! \brief Reimplemented from QAbstractTableModel.
    virtual bool setData( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
    
+   // Stuff for setting display units and scales -- per cell first, then by
+   // column
+
+   unitDisplay displayUnit(int column) const;
+   unitScale displayScale(int column) const;
+   void setDisplayUnit(int column, unitDisplay displayUnit);
+   void setDisplayScale(int column, unitScale displayScale);
+
+
+   QString generateName(int column) const;
+public slots:
+   void changed(QMetaProperty, QVariant);
+   //! \brief Add a hop to the model.
+   void addHop(Hop* hop);
+   //! \returns true if "hop" is successfully found and removed.
+   bool removeHop(Hop* hop);
+   
 private:
-   QVector<Hop*> hopObs;
+   QVector<Qt::ItemFlags> colFlags;
+   QList<Hop*> hopObs;
    Recipe* recObs;
-   HopTableWidget* parentTableWidget;
+   QTableView* parentTableWidget;
    bool showIBUs; // True if you want to show the IBU contributions in the table rows.
 };
 
+/*!
+ *  \class HopItemDelegate
+ *  \author Philip G. Lee
+ *
+ *  \brief An item delegate for hop tables.
+ *  \sa HopTableModel
+ */
 class HopItemDelegate : public QItemDelegate
 {
    Q_OBJECT
