@@ -1,6 +1,6 @@
 /*
  * InstructionWidget.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,22 +22,31 @@
 class InstructionWidget;
 
 #include "ui_instructionWidget.h"
-#include "observable.h"
-#include "instruction.h"
 #include <QWidget>
 #include <QSize>
-#include "TimerWidget.h"
+#include <QMetaProperty>
+#include <QVariant>
 
-class InstructionWidget : public QWidget, public Ui::instructionWidget, public Observer
+// Forward declarations.
+class TimerWidget;
+class Instruction;
+
+/*!
+ * \class InstructionWidget
+ * \author Philip G. Lee
+ *
+ * \brief View/controller widget that views/edits recipe instructions.
+ */
+class InstructionWidget : public QWidget, public Ui::instructionWidget
 {
    Q_OBJECT
 public:
    InstructionWidget(QWidget* parent=0);
    virtual ~InstructionWidget();
+   //! View/edit the given instruction.
    void setInstruction(Instruction* ins);
 
    virtual QSize sizeHint() const; // From QWidget
-   virtual void notify(Observable *notifier, QVariant info); // From Observer
 
 public slots:
    void setDirections();
@@ -45,6 +54,7 @@ public slots:
    void setTimerValue(QString value);
    void setCompleted();
 
+   void changed(QMetaProperty,QVariant);
 private:
    void showChanges();
    void makeEverythingInactive();
@@ -54,4 +64,3 @@ private:
 };
 
 #endif   /* _INGREDIENTWIDGET_H */
-
