@@ -1,6 +1,6 @@
 /*
 * MashComboBox.h is part of Brewtarget, and is Copyright Philip G. Lee
-* (rocketman768@gmail.com), 2009-2011.
+* (rocketman768@gmail.com), 2009-2013.
 *
 * Brewtarget is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -23,33 +23,51 @@ class MashComboBox;
 
 #include <QComboBox>
 #include <QWidget>
-#include <string>
-#include <QVector>
-#include "mash.h"
-#include "database.h"
-#include "observable.h"
+#include <QList>
+#include <QMetaProperty>
+#include <QVariant>
 
-class MashComboBox : public QComboBox, public MultipleObserver
+// Forward declaration.
+class Mash;
+
+/*!
+ * \class MashComboBox
+ * \author Philip G. Lee
+ *
+ * \brief A combobox that is a view class for a list of mashes.
+ *
+ * Well, it's not exactly
+ * a strict view class, since it contains model-related methods, so we should
+ * prune out the model methods at some point.
+ */
+class MashComboBox : public QComboBox
 {
   Q_OBJECT
   
    public:
       MashComboBox(QWidget* parent=0);
       virtual ~MashComboBox() {}
-      void startObservingDB();
-      void addMash(Mash* m);
-      void setIndexByMashName(QString name);
+      //! Set the current index to that which corresponds to \b m.
+      void setIndexByMash(Mash* m);
+      //! Set the index.
       void setIndex(int ndx);
+      //! Remove all mashs from the internal model.
       void removeAllMashs();
+      //! Populate the internal model with all the database mashs.
       void repopulateList();
       
+      //! \return the selected mash.
       Mash* getSelectedMash();
       
-      virtual void notify(Observable *notifier, QVariant info = QVariant()); // This will get called by observed whenever it changes.
+   public slots:
+      void changed(QMetaProperty, QVariant);
+      //! Add a mash to the internal model's list.
+      void addMash(Mash* m);
+      //! Remove a mash from the internal model's list.
+      void removeMash(Mash* m);
       
    private:
-      QVector<Mash*> mashObs;
-      Database* dbObs;
+      QList<Mash*> mashObs;
 };
 
 #endif
