@@ -1,6 +1,6 @@
 /*
  * MashEditor.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,9 +22,21 @@
 class MashEditor;
 
 #include <QDialog>
+#include <QMetaProperty>
+#include <QVariant>
 #include "ui_mashEditor.h"
-#include "recipe.h"
 
+// Forward declarations.
+class Recipe;
+class Mash;
+class Equipment;
+
+/*!
+ * \class MashEditor
+ * \author Philip G. Lee
+ *
+ * \brief View/controller dialog for editing a mash.
+ */
 class MashEditor : public QDialog, public Ui::mashEditor
 {
    Q_OBJECT
@@ -35,13 +47,18 @@ public slots:
    void showEditor();
    void closeEditor();
    void saveAndClose();
+   //! Get the tun mass and sp. heat from the equipment.
    void fromEquipment();
-   void setRecipe(Recipe* recipe);
-
+   //! Set the mash we wish to view/edit.
+   void setMash(Mash* mash);
+   void setEquipment(Equipment* e);
+   
+   void changed(QMetaProperty,QVariant);
 private:
-   void showChanges();
+   void showChanges(QMetaProperty* prop = 0);
    void clear();
-   Recipe* rec;
+   Mash* mashObs;
+   Equipment* equip;
 
 };
 
