@@ -1,6 +1,6 @@
 /*
  * MashStepEditor.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,27 +23,44 @@ class MashStepEditor;
 
 #include <QDialog>
 #include <QWidget>
+#include <QMetaProperty>
 #include <QVariant>
-#include "observable.h"
-#include "mashstep.h"
 #include "ui_mashStepEditor.h"
 
-class MashStepEditor : public QDialog, public Ui::mashStepEditor, public Observer
+// Forward declarations.
+class MashStep;
+
+/*!
+ * \class MashStepEditor
+ * \author Philip G. Lee
+ *
+ * \brief View/controller dialog for editing mash steps.
+ */
+class MashStepEditor : public QDialog, public Ui::mashStepEditor
 {
    Q_OBJECT
 public:
    MashStepEditor(QWidget* parent=0);
    virtual ~MashStepEditor() {}
-   virtual void notify(Observable *notifier, QVariant info=QVariant());
 
 public slots:
    void saveAndClose();
+   //! View/edit the given mash step.
    void setMashStep(MashStep* step);
    void close();
+   /*!
+    * Grays out irrelevant portions of the dialog.
+    * \param text - one of {"Infusion","Decoction","Temperature"} describing the mash step.
+    */
    void grayOutStuff(const QString& text);
+   void changed(QMetaProperty, QVariant);
 
 private:
-   void showChanges();
+   /*! Updates the UI elements effected by the \b metaProp of
+    *  the step we are watching. If \b metaProp is null,
+    *  then update all the UI elements at once.
+    */
+   void showChanges(QMetaProperty* metaProp = 0);
    void clear();
    MashStep* obs;
 };
