@@ -1,6 +1,6 @@
 /*
  * MashStepTableModel.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,31 +20,50 @@
 #include <QWidget>
 #include <QModelIndex>
 #include <QVariant>
-#include <Qt>
+#include <QTableView>
 #include <QItemDelegate>
 #include <QObject>
 #include <QComboBox>
 #include <QLineEdit>
 #include <QVector>
-#include <iostream>
+#include "database.h"
 #include "mashstep.h"
-#include "observable.h"
 #include "MashStepTableModel.h"
 #include "unit.h"
 #include "brewtarget.h"
 
-MashStepTableModel::MashStepTableModel(MashStepTableWidget* parent)
-: QAbstractTableModel(parent), Observer()
+MashStepTableModel::MashStepTableModel(QTableView* parent)
+   : QAbstractTableModel(parent), mashObs(0), parentTableWidget(parent)
 {
-   parentTableWidget = parent;
-   mashObs = 0;
 }
 
 void MashStepTableModel::setMash( Mash* m )
 {
+   int i;
+   if( mashObs )
+   {
+      beginRemoveRows( QModelIndex(), 0, steps.size()-1 );
+      // Remove mashObs and all steps.
+      disconnect( mashObs, 0, this, 0 );
+      for( i = 0; i < steps.size(); ++i )
+         disconnect( steps[i], 0, this, 0 );
+      steps.clear();
+      endRemoveRows();
+   }
+   
    mashObs = m;
-   setObserved(mashObs);
-   reset(); // Tell everybody that the table has changed.
+   if( mashObs )
+   {
+      QList<MashStep*> tmpSteps = mashObs->mashSteps();
+      beginInsertRows( QModelIndex(), 0, tmpSteps.size()-1 );
+      //connect( mashObs, SIGNAL(changed(QMetaProperty,QVariant)), this, SLOT(mashChanged(QMetaProperty,QVariant)) );
+      connect( mashObs, SIGNAL(mashStepsChanged()), this, SLOT(mashChanged()) );
+      steps = tmpSteps;
+      for( i = 0; i < steps.size(); ++i )
+         connect( steps[i], SIGNAL(changed(QMetaProperty,QVariant)), this, SLOT(mashStepChanged(QMetaProperty,QVariant)) );
+      endInsertRows();
+   }
+   //reset(); // Tell everybody that the table has changed.
    
    if( parentTableWidget )
    {
@@ -53,81 +72,28 @@ void MashStepTableModel::setMash( Mash* m )
    }
 }
 
-/***************************************************
-void MashStepTableModel::addMashStep(MashStep* step)
-{
-   QVector<MashStep*>::iterator iter;
-
-   //Check to see if it's already in the list
-   for( iter=stepObs.begin(); iter != stepObs.end(); iter++ )
-      if( *iter == step )
-         return;
-
-   stepObs.push_back(step);
-   addObserved(step);
-   reset(); // Tell everybody that the table has changed.
-
-   if( parentTableWidget )
-   {
-      parentTableWidget->resizeColumnsToContents();
-      parentTableWidget->resizeRowsToContents();
-   }
-}
-
-bool MashStepTableModel::removeMashStep(MashStep* step)
-{
-   QVector<MashStep*>::iterator iter;
-
-   for( iter=stepObs.begin(); iter != stepObs.end(); iter++ )
-      if( *iter == step )
-      {
-         stepObs.erase(iter);
-         removeObserved(step);
-         reset(); // Tell everybody the table has changed.
-         
-         if( parentTableWidget )
-         {
-            parentTableWidget->resizeColumnsToContents();
-            parentTableWidget->resizeRowsToContents();
-         }
-         
-         return true;
-      }
-
-   return false;
-}
-
-void MashStepTableModel::removeAll()
-{
-   unsigned int i;
-
-   for( i = 0; i < stepObs.size(); ++i )
-      removeObserved(stepObs[i]);
-
-   stepObs.clear();
-   reset();
-}
-
 MashStep* MashStepTableModel::getMashStep(unsigned int i)
 {
-   return stepObs[i];
+   if( i < static_cast<unsigned int>(steps.size()) )
+      return steps[i];
+   else
+      return 0;
 }
-*************************************************************/
 
-void MashStepTableModel::notify(Observable* notifier, QVariant info)
+void MashStepTableModel::mashChanged()
+{
+   // Remove and re-add all steps.
+   setMash( mashObs );
+}
+
+void MashStepTableModel::mashStepChanged(QMetaProperty prop, QVariant val)
 {
    int i;
-   bool ok = false;
-   
-   if( notifier != mashObs )
-      return;
-   
-   i = info.toInt(&ok); // mashObs' info says which MashStep changed.
-   
-   if( ok )
+   MashStep* stepSender = qobject_cast<MashStep*>(sender());
+   if( stepSender && (i = steps.indexOf(stepSender)) >= 0 )
    {
       emit dataChanged( QAbstractItemModel::createIndex(i, 0),
-         QAbstractItemModel::createIndex(i, MASHSTEPNUMCOLS));
+                        QAbstractItemModel::createIndex(i, MASHSTEPNUMCOLS-1));
    }
    else
       reset();
@@ -141,10 +107,7 @@ void MashStepTableModel::notify(Observable* notifier, QVariant info)
 
 int MashStepTableModel::rowCount(const QModelIndex& /*parent*/) const
 {
-   if( mashObs != 0 )
-      return mashObs->getNumMashSteps();
-   else
-      return 0;
+   return steps.size();
 }
 
 int MashStepTableModel::columnCount(const QModelIndex& /*parent*/) const
@@ -155,41 +118,50 @@ int MashStepTableModel::columnCount(const QModelIndex& /*parent*/) const
 QVariant MashStepTableModel::data( const QModelIndex& index, int role ) const
 {
    MashStep* row;
+   unitDisplay unit;
+   unitScale scale;
+   int col = index.column();
 
    if( mashObs == 0 )
       return QVariant();
    
    // Ensure the row is ok.
-   if( index.row() >= (int)(mashObs->getNumMashSteps()) )
+   if( index.row() >= (int)(steps.size()) )
    {
       Brewtarget::log(Brewtarget::WARNING, tr("Bad model index. row = %1").arg(index.row()));
       return QVariant();
    }
    else
-      row = mashObs->getMashStep(index.row());
+      row = steps[index.row()];
 
    // Make sure we only respond to the DisplayRole role.
    if( role != Qt::DisplayRole )
       return QVariant();
 
-   switch( index.column() )
+   switch( col )
    {
       case MASHSTEPNAMECOL:
-         return QVariant(row->getName());
+         return QVariant(row->name());
       case MASHSTEPTYPECOL:
-         return QVariant(row->getTypeStringTr());
+         return QVariant(row->typeStringTr());
       case MASHSTEPAMOUNTCOL:
-         return (row->getType() == MashStep::TYPEDECOCTION)
-                ? QVariant( Brewtarget::displayAmount(row->getDecoctionAmount_l(), Units::liters ) )
-                : QVariant( Brewtarget::displayAmount(row->getInfuseAmount_l(), Units::liters) );
+
+         unit = displayUnit(col);
+         scale = displayScale(col);
+
+         return (row->type() == MashStep::Decoction)
+                ? QVariant( Brewtarget::displayAmount(row->decoctionAmount_l(), Units::liters, 3, unit, scale ) )
+                : QVariant( Brewtarget::displayAmount(row->infuseAmount_l(), Units::liters, 3, unit, scale) );
       case MASHSTEPTEMPCOL:
-         return (row->getType() == MashStep::TYPEDECOCTION)
+         unit = displayUnit(col);
+         return (row->type() == MashStep::Decoction)
                 ? QVariant("---")
-                : QVariant( Brewtarget::displayAmount(row->getInfuseTemp_c(), Units::celsius) );
+                : QVariant( Brewtarget::displayAmount(row->infuseTemp_c(), Units::celsius, 3, unit, noScale) );
       case MASHSTEPTARGETTEMPCOL:
-         return QVariant( Brewtarget::displayAmount(row->getStepTemp_c(), Units::celsius) );
+         unit = displayUnit(col);
+         return QVariant( Brewtarget::displayAmount(row->stepTemp_c(), Units::celsius,3, unit, noScale) );
       case MASHSTEPTIMECOL:
-         return QVariant( Brewtarget::displayAmount(row->getStepTime_min(), Units::minutes) );
+         return QVariant( Brewtarget::displayAmount(row->stepTime_min(), Units::minutes) );
       default :
          Brewtarget::log(Brewtarget::WARNING, tr("Bad column: %1").arg(index.column()));
          return QVariant();
@@ -238,14 +210,15 @@ Qt::ItemFlags MashStepTableModel::flags(const QModelIndex& index ) const
 bool MashStepTableModel::setData( const QModelIndex& index, const QVariant& value, int role )
 {
    MashStep *row;
+   unitDisplay unit;
 
    if( mashObs == 0 )
       return false;
    
-   if( index.row() >= (int)(mashObs->getNumMashSteps()) || role != Qt::EditRole )
+   if( index.row() >= (int)(steps.size()) || role != Qt::EditRole )
       return false;
    else
-      row = mashObs->getMashStep(index.row());
+      row = steps[index.row()];
 
    switch( index.column() )
    {
@@ -268,18 +241,20 @@ bool MashStepTableModel::setData( const QModelIndex& index, const QVariant& valu
       case MASHSTEPAMOUNTCOL:
          if( value.canConvert(QVariant::String) )
          {
-            if( row->getType() == MashStep::TYPEDECOCTION )
-               row->setDecoctionAmount_l( Brewtarget::volQStringToSI(value.toString()) );
+            unit = displayUnit(MASHSTEPAMOUNTCOL);
+            if( row->type() == MashStep::Decoction )
+               row->setDecoctionAmount_l( Brewtarget::volQStringToSI(value.toString(),unit) );
             else
-               row->setInfuseAmount_l( Brewtarget::volQStringToSI(value.toString()) );
+               row->setInfuseAmount_l( Brewtarget::volQStringToSI(value.toString(),unit) );
             return true;
          }
          else
             return false;
       case MASHSTEPTEMPCOL:
-         if( value.canConvert(QVariant::String) && row->getType() != MashStep::TYPEDECOCTION )
+         if( value.canConvert(QVariant::String) && row->type() != MashStep::Decoction )
          {
-            row->setInfuseTemp_c( Brewtarget::tempQStringToSI(value.toString()) );
+            unit = displayUnit(MASHSTEPTEMPCOL);
+            row->setInfuseTemp_c( Brewtarget::tempQStringToSI(value.toString(),unit) );
             return true;
          }
          else
@@ -287,8 +262,9 @@ bool MashStepTableModel::setData( const QModelIndex& index, const QVariant& valu
       case MASHSTEPTARGETTEMPCOL:
          if( value.canConvert(QVariant::String) )
          {
-            row->setStepTemp_c( Brewtarget::tempQStringToSI(value.toString()) );
-            row->setEndTemp_c( Brewtarget::tempQStringToSI(value.toString()) );
+            unit = displayUnit(MASHSTEPTARGETTEMPCOL);
+            row->setStepTemp_c( Brewtarget::tempQStringToSI(value.toString(),unit) );
+            row->setEndTemp_c( Brewtarget::tempQStringToSI(value.toString(),unit) );
             return true;
          }
          else
@@ -306,21 +282,106 @@ bool MashStepTableModel::setData( const QModelIndex& index, const QVariant& valu
    }
 }
 
-void MashStepTableModel::moveStepUp(unsigned int i)
+void MashStepTableModel::moveStepUp(int i)
 {
-   if( mashObs == 0 || i == 0 || i >= mashObs->getNumMashSteps() )
+   if( mashObs == 0 || i == 0 || i >= steps.size() )
       return;
 
-   mashObs->swapSteps(i, i-1);
+   Database::instance().swapMashStepOrder( steps[i], steps[i-1] );
 }
 
-void MashStepTableModel::moveStepDown(unsigned int i)
+void MashStepTableModel::moveStepDown(int i)
 {
-   // i is an unsigned int. How can i be less than 0?
-   if( mashObs == 0 || i < 0 || i+1 >= mashObs->getNumMashSteps() )
+   if( mashObs == 0 ||  i+1 >= steps.size() )
       return;
 
-   mashObs->swapSteps(i, i+1);
+   Database::instance().swapMashStepOrder( steps[i], steps[i+1] );
+}
+
+unitDisplay MashStepTableModel::displayUnit(int column) const
+{ 
+   QString attribute = generateName(column);
+
+   if ( attribute.isEmpty() )
+      return noUnit;
+
+   return (unitDisplay)Brewtarget::option(attribute, noUnit, this, Brewtarget::UNIT).toInt();
+}
+
+unitScale MashStepTableModel::displayScale(int column) const
+{ 
+   QString attribute = generateName(column);
+
+   if ( attribute.isEmpty() )
+      return noScale;
+
+   return (unitScale)Brewtarget::option(attribute, noScale, this, Brewtarget::SCALE).toInt();
+}
+
+// We need to:
+//   o clear the custom scale if set
+//   o clear any custom unit from the rows
+//      o which should have the side effect of clearing any scale
+void MashStepTableModel::setDisplayUnit(int column, unitDisplay displayUnit) 
+{
+   // MashStep* row; // disabled per-cell magic
+   QString attribute = generateName(column);
+
+   if ( attribute.isEmpty() )
+      return;
+
+   Brewtarget::setOption(attribute,displayUnit,this,Brewtarget::UNIT); 
+   Brewtarget::setOption(attribute,noScale,this,Brewtarget::SCALE);
+
+   /* Disabled cell-specific code
+   for (int i = 0; i < rowCount(); ++i )
+   {
+      row = getMashStep(i);
+      row->setDisplayUnit(noUnit);
+   }
+   */
+}
+
+// Setting the scale should clear any cell-level scaling options
+void MashStepTableModel::setDisplayScale(int column, unitScale displayScale) 
+{ 
+   // MashStep* row; //disabled per-cell magic
+
+   QString attribute = generateName(column);
+
+   if ( attribute.isEmpty() )
+      return;
+
+   Brewtarget::setOption(attribute,displayScale,this,Brewtarget::SCALE); 
+
+   /* disabled cell-specific code
+   for (int i = 0; i < rowCount(); ++i )
+   {
+      row = getMashStep(i);
+      row->setDisplayScale(noScale);
+   }
+   */
+}
+
+QString MashStepTableModel::generateName(int column) const
+{
+   QString attribute;
+
+   switch(column)
+   {
+      case MASHSTEPAMOUNTCOL:
+         attribute = "amount";
+         break;
+      case MASHSTEPTEMPCOL:
+         attribute = "infuseTemp_c";
+         break;
+      case MASHSTEPTARGETTEMPCOL:
+         attribute = "stepTemp_c";
+         break;
+      default:
+         attribute = "";
+   }
+   return attribute;
 }
 
 //==========================CLASS MashStepItemDelegate===============================
@@ -351,7 +412,7 @@ void MashStepItemDelegate::setEditorData(QWidget *editor, const QModelIndex &ind
 {
    if( index.column() == MASHSTEPTYPECOL )
    {
-      QComboBox* box = (QComboBox*)editor;
+      QComboBox* box = qobject_cast<QComboBox*>(editor);
       QString text = index.model()->data(index, Qt::DisplayRole).toString();
 
       int index = box->findText(text);
@@ -359,7 +420,7 @@ void MashStepItemDelegate::setEditorData(QWidget *editor, const QModelIndex &ind
    }
    else
    {
-      QLineEdit* line = (QLineEdit*)editor;
+      QLineEdit* line = qobject_cast<QLineEdit*>(editor);
 
       line->setText(index.model()->data(index, Qt::DisplayRole).toString());
    }
@@ -368,18 +429,22 @@ void MashStepItemDelegate::setEditorData(QWidget *editor, const QModelIndex &ind
 
 void MashStepItemDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
 {
+   QStringList typesTr = QStringList() << QObject::tr("Infusion") << QObject::tr("Temperature") << QObject::tr("Decoction");
    if( index.column() == MASHSTEPTYPECOL )
    {
-      QComboBox* box = (QComboBox*)editor;
-      QString value = box->currentText();
+      QComboBox* box = qobject_cast<QComboBox*>(editor);
+      int ndx = box->currentIndex();
+      int curr  = typesTr.indexOf(model->data(index,Qt::DisplayRole).toString());
 
-      model->setData(index, value, Qt::EditRole);
+      if ( ndx != curr )
+         model->setData(index, ndx, Qt::EditRole);
    }
    else
    {
-      QLineEdit* line = (QLineEdit*)editor;
+      QLineEdit* line = qobject_cast<QLineEdit*>(editor);
 
-      model->setData(index, line->text(), Qt::EditRole);
+      if ( line->isModified() )
+         model->setData(index, line->text(), Qt::EditRole);
    }
 }
 
@@ -387,4 +452,3 @@ void MashStepItemDelegate::updateEditorGeometry(QWidget *editor, const QStyleOpt
 {
    editor->setGeometry(option.rect);
 }
-
