@@ -1,6 +1,6 @@
 /*
  * MashStepTableModel.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,46 +26,73 @@ class MashStepItemDelegate;
 #include <QWidget>
 #include <QModelIndex>
 #include <QVariant>
-#include <Qt>
+#include <QMetaProperty>
+#include <QVariant>
 #include <QItemDelegate>
 #include <QVector>
+#include <QTableView>
+
 #include "mashstep.h"
 #include "mash.h"
-#include "observable.h"
-#include "MashStepTableWidget.h"
+#include "unit.h"
 
 enum{ MASHSTEPNAMECOL, MASHSTEPTYPECOL, MASHSTEPAMOUNTCOL, MASHSTEPTEMPCOL, MASHSTEPTARGETTEMPCOL, MASHSTEPTIMECOL, MASHSTEPNUMCOLS /*This one MUST be last*/};
 
-class MashStepTableModel : public QAbstractTableModel, public Observer
+/*!
+ * \class MashStepTableModel
+ * \author Philip G. Lee
+ *
+ * \brief Model for the list of mash steps in a mash.
+ */
+class MashStepTableModel : public QAbstractTableModel
 {
    Q_OBJECT
 
 public:
-   MashStepTableModel(MashStepTableWidget* parent=0);
+   MashStepTableModel(QTableView* parent=0);
    virtual ~MashStepTableModel() {}
+   //! Set the mash whose mash steps we want to model.
    void setMash( Mash* m );
-   //void addMashStep(MashStep* step);
-   //bool removeMashStep(MashStep* step); // Returns true if "step" is successfully found and removed.
-   //void removeAll();
-   //MashStep* getMashStep(unsigned int i);
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer via MultipleObserver.
+   //! \returns the mash step at model index \b i.
+   MashStep* getMashStep(unsigned int i);
 
-   void moveStepUp(unsigned int i);
-   void moveStepDown(unsigned int i);
-
-   // Inherit the following from QAbstractItemModel via QAbstractTableModel
+   //! Reimplemented from QAbstractTableModel.
    virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
+   //! Reimplemented from QAbstractTableModel.
    virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
+   //! Reimplemented from QAbstractTableModel.
    virtual QVariant data( const QModelIndex& index, int role = Qt::DisplayRole ) const;
+   //! Reimplemented from QAbstractTableModel.
    virtual QVariant headerData( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
+   //! Reimplemented from QAbstractTableModel.
    virtual Qt::ItemFlags flags(const QModelIndex& index ) const;
+   //! Reimplemented from QAbstractTableModel.
    virtual bool setData( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
 
+   unitDisplay displayUnit(int column) const;
+   unitScale displayScale(int column) const;
+   void setDisplayUnit(int column, unitDisplay displayUnit);
+   void setDisplayScale(int column, unitScale displayScale);
+   QString generateName(int column) const;
+
+public slots:
+   void moveStepUp(int i);
+   void moveStepDown(int i);
+   void mashChanged();
+   void mashStepChanged(QMetaProperty,QVariant);
+   
 private:
    Mash* mashObs;
-   MashStepTableWidget* parentTableWidget;
+   QTableView* parentTableWidget;
+   QList<MashStep*> steps;
 };
 
+/*!
+ * \class MashStepItemDelegate
+ * \author Philip G. Lee
+ *
+ * An item delegate for mash step tables.
+ */
 class MashStepItemDelegate : public QItemDelegate
 {
    Q_OBJECT
