@@ -1,6 +1,6 @@
 /*
  * MiscDialog.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,36 +20,41 @@
 #include <QDialog>
 #include <QInputDialog>
 #include <QString>
-#include <string>
 #include <QList>
 #include "MiscDialog.h"
-#include "observable.h"
 #include "database.h"
 #include "recipe.h"
 #include "MainWindow.h"
 #include "misc.h"
 #include "MiscEditor.h"
 #include "MiscTableModel.h"
+#include "MiscSortFilterProxyModel.h"
 
 MiscDialog::MiscDialog(MainWindow* parent)
-        : QDialog(parent)
+        : QDialog(parent), mainWindow(parent), numMiscs(0), miscEdit(new MiscEditor(this))
 {
    setupUi(this);
-   mainWindow = parent;
-   dbObs = 0;
-   numMiscs = 0;
-   miscEdit = new MiscEditor(this);
 
+   miscTableModel = new MiscTableModel(miscTableWidget, false);
+   miscTableProxy = new MiscSortFilterProxyModel(miscTableWidget);
+   miscTableProxy->setSourceModel(miscTableModel);
+   miscTableWidget->setModel(miscTableProxy);
+   miscTableWidget->setSortingEnabled(true);
+   miscTableWidget->sortByColumn( MISCNAMECOL, Qt::AscendingOrder );
+   miscTableProxy->setDynamicSortFilter(true);
+   
    connect( pushButton_addToRecipe, SIGNAL( clicked() ), this, SLOT( addMisc() ) );
    connect( pushButton_new, SIGNAL(clicked()), this, SLOT( newMisc() ) );
    connect( pushButton_edit, SIGNAL(clicked()), this, SLOT(editSelected()) );
    connect( pushButton_remove, SIGNAL(clicked()), this, SLOT(removeMisc()) );
    connect( miscTableWidget, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT( addMisc(const QModelIndex&) ) );
+   
+   miscTableModel->observeDatabase(true);
 }
 
 void MiscDialog::removeMisc()
 {
-   QModelIndexList selected = miscTableWidget->selectedIndexes();
+   QModelIndexList selected = miscTableWidget->selectionModel()->selectedIndexes();
    int row, size, i;
 
    size = selected.size();
@@ -64,37 +69,8 @@ void MiscDialog::removeMisc()
          return;
    }
 
-   Misc* m = miscTableWidget->getModel()->getMisc(row);
-   dbObs->removeMisc(m);
-}
-
-void MiscDialog::notify(Observable *notifier, QVariant info)
-{
-   if( notifier != dbObs || (info.toInt() != DBMISC && info.toInt() != DBALL) )
-      return;
-
-   miscTableWidget->getModel()->removeAll();
-   populateTable();
-}
-
-void MiscDialog::startObservingDB()
-{
-   dbObs = Database::getDatabase();
-   setObserved(dbObs);
-   populateTable();
-}
-
-void MiscDialog::populateTable()
-{
-   QList<Misc*>::iterator it, end;
-
-   if( ! Database::isInitialized() )
-      return;
-
-   numMiscs = dbObs->getNumMiscs();
-   end = dbObs->getMiscEnd();
-   for( it = dbObs->getMiscBegin(); it != end; ++it )
-      miscTableWidget->getModel()->addMisc(*it);
+   Misc* m = miscTableModel->getMisc(miscTableProxy->mapToSource(selected[0]).row());
+   Database::instance().removeMisc(m);
 }
 
 void MiscDialog::addMisc(const QModelIndex& index)
@@ -103,7 +79,7 @@ void MiscDialog::addMisc(const QModelIndex& index)
    
    if( !index.isValid() )
    {
-      QModelIndexList selected = miscTableWidget->selectedIndexes();
+      QModelIndexList selected = miscTableWidget->selectionModel()->selectedIndexes();
       int row, size, i;
 
       size = selected.size();
@@ -117,8 +93,9 @@ void MiscDialog::addMisc(const QModelIndex& index)
          if( selected[i].row() != row )
             return;
       }
-      
-      translated = selected[0];
+     
+      // Always need to translate indices through the proxy 
+      translated = miscTableProxy->mapToSource(selected[0]);
    }
    else
    {
@@ -126,18 +103,19 @@ void MiscDialog::addMisc(const QModelIndex& index)
       // this keeps us from adding something to the recipe when we just want to edit
       // one of the other columns.
       if( index.column() == MISCNAMECOL )
-         translated = miscTableWidget->getProxy()->mapToSource(index);
+         translated = miscTableProxy->mapToSource(index);
       else
          return;
    }
    
-   Misc *misc = miscTableWidget->getModel()->getMisc(translated.row());
-   mainWindow->addMiscToRecipe(new Misc(*misc) ); // Need to add a copy so we don't change the database.
+   Misc *misc = miscTableModel->getMisc(translated.row());
+   
+   Database::instance().addToRecipe( mainWindow->currentRecipe(), misc );
 }
 
 void MiscDialog::editSelected()
 {
-   QModelIndexList selected = miscTableWidget->selectedIndexes();
+   QModelIndexList selected = miscTableWidget->selectionModel()->selectedIndexes();
    int row, size, i;
 
    size = selected.size();
@@ -152,7 +130,7 @@ void MiscDialog::editSelected()
          return;
    }
 
-   Misc* m = miscTableWidget->getModel()->getMisc(row);
+   Misc* m = miscTableModel->getMisc(miscTableProxy->mapToSource(selected[0]).row());
    miscEdit->setMisc(m);
    miscEdit->show();
 }
@@ -164,11 +142,8 @@ void MiscDialog::newMisc()
    if(name.isEmpty())
       return;
 
-   Misc *m = new Misc();
-   QString stdname = name;
-   m->setName(stdname);
-
-   dbObs->addMisc(m);
+   Misc* m = Database::instance().newMisc();
+   m->setName(name);
    miscEdit->setMisc(m);
    miscEdit->show();
 }
