@@ -1,6 +1,6 @@
 /*
  * MiscDialog.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,34 +25,45 @@ class MiscDialog;
 #include <QDialog>
 #include <QVariant>
 #include "ui_miscDialog.h"
-#include "observable.h"
 #include "database.h"
-class MainWindow;
-#include "MiscEditor.h"
 
-class MiscDialog : public QDialog, public Ui::miscDialog, public Observer
+// Forward declarations.
+class MainWindow;
+class MiscEditor;
+class MiscTableModel;
+class MiscSortFilterProxyModel;
+
+/*!
+ * \class MiscDialog
+ * \author Philip G. Lee
+ *
+ * \brief View/controller dialog for the miscs in the database.
+ */
+class MiscDialog : public QDialog, public Ui::miscDialog
 {
    Q_OBJECT
 
 public:
    MiscDialog(MainWindow* parent);
    virtual ~MiscDialog() {}
-   void startObservingDB();
-   virtual void notify(Observable *notifier, QVariant info = QVariant()); // From Observer
 
 public slots:
+   //! Add the selected misc to the current recipe.
    void addMisc(const QModelIndex& = QModelIndex());
+   //! Delete selected misc from the database.
    void removeMisc();
+   //! Bring up the editor for the selected misc.
    void editSelected();
+   //! Add a new misc to the database.
    void newMisc();
-
+   
 private:
-   Database* dbObs;
    MainWindow* mainWindow;
-   unsigned int numMiscs;
+   MiscTableModel* miscTableModel;
+   MiscSortFilterProxyModel* miscTableProxy;
+   int numMiscs;
    MiscEditor* miscEdit;
 
-   void populateTable();
 };
 
 #endif   /* _MISCDIALOG_H */
