@@ -1,6 +1,6 @@
 /*
  * MiscEditor.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,28 +19,44 @@
 #ifndef _MISCEDITOR_H
 #define   _MISCEDITOR_H
 
+#include <QDialog>
 #include "ui_miscEditor.h"
-#include "misc.h"
-#include "observable.h"
+#include <QMetaProperty>
+#include <QVariant>
 
-class MiscEditor : public QDialog, private Ui::miscEditor, Observer
+// Forward declarations.
+class Misc;
+
+/*!
+ * \class MiscEditor
+ * \author Philip G. Lee
+ *
+ * \brief View/controller dialog for editing miscs.
+ */
+class MiscEditor : public QDialog, private Ui::miscEditor
 {
    Q_OBJECT
 
 public:
    MiscEditor( QWidget *parent=0 );
    virtual ~MiscEditor() {}
+   //! Set the misc we wish to view/edit.
    void setMisc( Misc* m );
    
 public slots:
+   //! Save changes.
    void save();
+   //! Clear dialog and close.
    void clearAndClose();
+   void changed(QMetaProperty,QVariant);
    
 private:
    Misc* obsMisc;
-   
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer
-   void showChanges();
+   /*! Updates the UI elements effected by the \b metaProp of
+    *  the misc we are watching. If \b metaProp is null,
+    *  then update all the UI elements at once.
+    */
+   void showChanges(QMetaProperty* metaProp = 0);
 };
 
 #endif   /* _MISCEDITOR_H */
