@@ -1,6 +1,6 @@
 /*
  * MiscSortFilterProxyModel.cpp is part of Brewtarget, and is Copyright
- * Philip G. Lee <rocketman768@gmail.com>, 2011.
+ * Philip G. Lee <rocketman768@gmail.com>, 2011-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,16 +23,26 @@ class MiscSortFilterProxyModel;
 
 #include <QSortFilterProxyModel>
 
+/*!
+ * \class MiscSortFilterProxyModel
+ * \author Philip G. Lee
+ *
+ * \brief Proxy model for sorting miscs.
+ */
 class MiscSortFilterProxyModel : public QSortFilterProxyModel
 {
    Q_OBJECT
 
 public:
-   MiscSortFilterProxyModel(QObject *parent = 0);
+   MiscSortFilterProxyModel(QObject *parent = 0, bool filt = true);
 
 protected:
-   bool lessThan(const QModelIndex &left, const QModelIndex &right) const;
 
+   bool lessThan(const QModelIndex &left, const QModelIndex &right) const;
+   bool filterAcceptsRow( int source_row, const QModelIndex &source_parent) const;
+
+private:
+   bool filter;
 };
 
 #endif
