@@ -1,6 +1,6 @@
 /*
  * MiscTableModel.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  */
 
 #ifndef _MISCTABLEMODEL_H
-#define   _MISCTABLEMODEL_H
+#define _MISCTABLEMODEL_H
 
 class MiscTableModel;
 class MiscItemDelegate;
@@ -27,42 +27,88 @@ class MiscItemDelegate;
 #include <QWidget>
 #include <QModelIndex>
 #include <QVariant>
-#include <Qt>
 #include <QItemDelegate>
 #include <QStyleOptionViewItem>
-#include <QVector>
-#include "misc.h"
-#include "observable.h"
-#include "MiscTableWidget.h"
+#include <QList>
+#include <QMetaProperty>
+#include <QTableView>
 
-enum{MISCNAMECOL, MISCTYPECOL, MISCUSECOL, MISCTIMECOL, MISCAMOUNTCOL, MISCNUMCOLS /*This one MUST be last*/};
+#include "unit.h"
 
-class MiscTableModel : public QAbstractTableModel, public MultipleObserver
+// Forward declarations.
+class Misc;
+class MiscTableWidget;
+class Recipe;
+
+enum{MISCNAMECOL, MISCTYPECOL, MISCUSECOL, MISCTIMECOL, MISCAMOUNTCOL, MISCISWEIGHT, MISCNUMCOLS /*This one MUST be last*/};
+
+/*!
+ * \class MiscTableModel
+ * \author Philip G. Lee
+ *
+ * \brief Table model for a list of miscs.
+ */
+class MiscTableModel : public QAbstractTableModel
 {
    Q_OBJECT
    
 public:
-   MiscTableModel(MiscTableWidget* parent=0);
+   MiscTableModel(QTableView* parent=0, bool editable=true);
    virtual ~MiscTableModel() {}
-   void addMisc(Misc* misc);
+   //! \brief Observe a recipe's list of miscs.
+   void observeRecipe(Recipe* rec);
+   //! \brief If true, we model the database's list of miscs.
+   void observeDatabase(bool val);
+   //! \brief Add \c miscs to the model.
+   void addMiscs(QList<Misc*> miscs);
+   //! \returns the \c Misc at model index \b i.
    Misc* getMisc(unsigned int i);
-   bool removeMisc(Misc* misc);
+   //! \brief Clear the model.
    void removeAll();
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer via MultipleObserver
    
-   // Inherit the following from QAbstractItemModel via QAbstractTableModel
+   //! \brief Reimplemented from QAbstractTableModel
    virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
+   //! \brief Reimplemented from QAbstractTableModel
    virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
+   //! \brief Reimplemented from QAbstractTableModel
    virtual QVariant data( const QModelIndex& index, int role = Qt::DisplayRole ) const;
+   //! \brief Reimplemented from QAbstractTableModel
    virtual QVariant headerData( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
+   //! \brief Reimplemented from QAbstractTableModel
    virtual Qt::ItemFlags flags(const QModelIndex& index ) const;
+   //! \brief Reimplemented from QAbstractTableModel
    virtual bool setData( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
-   
+
+   unitDisplay displayUnit(int column) const;
+   unitScale displayScale(int column) const;
+   void setDisplayUnit(int column, unitDisplay displayUnit);
+   void setDisplayScale(int column, unitScale displayScale);
+   QString generateName(int column) const;
+
+public slots:
+   //! \brief Add a misc to the model.
+   void addMisc(Misc* misc);
+   //! \brief Remove a misc from the model.
+   bool removeMisc(Misc* misc);
+
+private slots:
+   //! \brief Catch changes to Recipe, Database, and Misc.
+   void changed(QMetaProperty, QVariant);
+
 private:
-   QVector<Misc*> miscObs;
-   MiscTableWidget* parentTableWidget;
+   bool editable;
+   QList<Misc*> miscObs;
+   Recipe* recObs;
+   QTableView* parentTableWidget;
 };
 
+/*!
+ *  \class MiscItemDelegate
+ *  \author Philip G. Lee
+ *
+ *  \brief Item delegate for misc tables.
+ *  \sa MiscTableModel
+ */
 class MiscItemDelegate : public QItemDelegate
 {
    Q_OBJECT
@@ -80,4 +126,3 @@ private:
 };
 
 #endif   /* _MISCTABLEMODEL_H */
-
