@@ -1,6 +1,6 @@
 /*
  * OptionDialog.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,17 +31,17 @@
 #include <QMessageBox>
 #include <QFileInfo>
 #include <QFileDialog>
+#include "MainWindow.h"
 
 OptionDialog::OptionDialog(QWidget* parent)
 {
+   int i;
    setupUi(this);
 
    if( parent != 0 )
    {
       setWindowIcon(parent->windowIcon());
    }
-
-   languageGroup = new QButtonGroup(this);
    colorGroup = new QButtonGroup(this);
    ibuGroup = new QButtonGroup(this);
    weightGroup = new QButtonGroup(this);
@@ -50,19 +50,46 @@ OptionDialog::OptionDialog(QWidget* parent)
    gravGroup = new QButtonGroup(this);
    colorUnitGroup = new QButtonGroup(this);
 
-   // Set up language map.
-   languageToButtonMap["ca"] = pushButton_ca;
-   languageToButtonMap["cs"] = pushButton_cs;
-   languageToButtonMap["de"] = pushButton_de;
-   languageToButtonMap["en"] = pushButton_en;
-   languageToButtonMap["es"] = pushButton_es;
-   languageToButtonMap["fr"] = pushButton_fr;
-   languageToButtonMap["nl"] = pushButton_nl;
-   languageToButtonMap["pl"] = pushButton_pl;
-   languageToButtonMap["pt"] = pushButton_pt;
-
+   ndxToLangCode <<
+      "ca" <<
+      "cs" <<
+      "de" <<
+      "en" <<
+      "el" <<
+      "es" <<
+      "fr" <<
+      "it" <<
+      "nl" <<
+      "pl" <<
+      "pt" <<
+      "ru" <<
+      "zh";
+   
+   // Do this just to have model indices to set icons.
+   comboBox_lang->addItems(ndxToLangCode);
+   // MUST correspond to ndxToLangCode.
+   langIcons <<
+      QIcon(":images/flagCatalonia.svg") <<
+      QIcon(":images/flagCzech.svg") <<
+      QIcon(":images/flagGermany.svg") <<
+      QIcon(":images/flagUK.svg") <<
+      QIcon(":images/flagGreece.svg") <<
+      QIcon(":images/flagSpain.svg") <<
+      QIcon(":images/flagFrance.svg") <<
+      QIcon(":images/flagItaly.svg") <<
+      QIcon(":images/flagNetherlands.svg") <<
+      QIcon(":images/flagPoland.svg") <<
+      QIcon(":images/flagPortugal.svg") <<
+      QIcon(":images/flagRussia.svg") <<
+      QIcon(":images/flagChina.svg");
+   // Set icons.
+   for( i = 0; i < langIcons.size(); ++i )
+      comboBox_lang->setItemIcon(i, langIcons[i]);
+   
+   // Call this here to set up translatable strings.
+   retranslate();
+   
    // Want you to only be able to select exactly one in each group.
-   languageGroup->setExclusive(true);
    colorGroup->setExclusive(true);
    ibuGroup->setExclusive(true);
    weightGroup->setExclusive(true);
@@ -71,20 +98,6 @@ OptionDialog::OptionDialog(QWidget* parent)
    gravGroup->setExclusive(true);
    colorUnitGroup->setExclusive(true);
 
-   // Disable certain buttons due to lack of actual language support.
-   pushButton_de->setEnabled(false);
-
-   // Set up the buttons in languageGroup;
-   languageGroup->addButton(pushButton_ca);
-   languageGroup->addButton(pushButton_cs);
-   languageGroup->addButton(pushButton_de);
-   languageGroup->addButton(pushButton_en);
-   languageGroup->addButton(pushButton_es);
-   languageGroup->addButton(pushButton_fr);
-   languageGroup->addButton(pushButton_nl);
-   languageGroup->addButton(pushButton_pl);
-   languageGroup->addButton(pushButton_pt);
-
    // Set up the buttons in the colorGroup
    colorGroup->addButton(checkBox_mosher);
    colorGroup->addButton(checkBox_daniel);
@@ -115,19 +128,40 @@ OptionDialog::OptionDialog(QWidget* parent)
    // Color Unit
    colorUnitGroup->addButton(radioButton_srm);
    colorUnitGroup->addButton(radioButton_ebc);
-
-   //connect( colorGroup, SIGNAL( buttonClicked(QAbstractButton*) ), this, SLOT( changeColorFormula(QAbstractButton*) ) );
-   //connect( ibuGroup, SIGNAL( buttonClicked(QAbstractButton*) ), this, SLOT( changeIbuFormula(QAbstractButton*) ) );
-   //connect( weightGroup, SIGNAL( buttonClicked(QAbstractButton*) ), this, SLOT( changeWeightUnitSystem(QAbstractButton*) ) );
-   //connect( volumeGroup, SIGNAL( buttonClicked(QAbstractButton*) ), this, SLOT( changeVolumeUnitSystem(QAbstractButton*) ) );
-   //connect( tempGroup, SIGNAL( buttonClicked(QAbstractButton*) ), this, SLOT( changeTemperatureScale(QAbstractButton*) ) );
-
+   
    connect( buttonBox, SIGNAL( accepted() ), this, SLOT( saveAndClose() ) );
    connect( buttonBox, SIGNAL( rejected() ), this, SLOT( cancel() ) );
    connect( pushButton_dbDirBrowse, SIGNAL( clicked() ), this, SLOT( setDataDir() ) );
    connect( pushButton_dbDirDefault, SIGNAL( clicked() ), this, SLOT( defaultDataDir() ) );
 }
 
+void OptionDialog::retranslate()
+{
+   // Let the Ui take care of its business
+   retranslateUi(this);
+   
+   // Retranslate the language combobox.
+   // NOTE: the indices MUST correspond to ndxToLangCode.
+   QStringList langStrings;
+   langStrings <<
+      tr("Catalan") <<
+      tr("Czech") <<
+      tr("German") <<
+      tr("English") <<
+      tr("Greek") <<
+      tr("Spanish") <<
+      tr("French") <<
+      tr("Italian") <<
+      tr("Dutch") <<
+      tr("Polish") <<
+      tr("Portuguese") <<
+      tr("Russian") <<
+      tr("Chinese");
+   int i;
+   for( i = 0; i < langStrings.size(); ++i )
+      comboBox_lang->setItemText(i, langStrings[i]);
+}
+
 void OptionDialog::show()
 {
    showChanges();
@@ -148,11 +182,6 @@ void OptionDialog::defaultDataDir()
 
 void OptionDialog::saveAndClose()
 {
-   /*
-   Brewtarget::weightUnitSystem = weightUnitSystem;
-   Brewtarget::volumeUnitSystem = volumeUnitSystem;
-   Brewtarget::tempScale = temperatureScale;
-   */
    QAbstractButton* button;
    iUnitSystem weightUnitSystem;
    iUnitSystem volumeUnitSystem;
@@ -249,24 +278,24 @@ void OptionDialog::saveAndClose()
    Brewtarget::colorUnit = colorUnit;
 
    // Set the right language.
-   Brewtarget::setLanguage( languageToButtonMap.key(qobject_cast<QPushButton*>(languageGroup->checkedButton())) );
+   Brewtarget::setLanguage( ndxToLangCode[ comboBox_lang->currentIndex() ] );
    
    // Check the new userDataDir.
    newUserDataDir = lineEdit_dbDir->text();
 
-	 // Make sure the dir ends with a "/" or "\"
+   // Make sure the dir ends with a "/" or "\"
 #if defined(Q_WS_X11) || defined(Q_WS_MAC)
-	 if( !newUserDataDir.endsWith("/") )
-		 newUserDataDir += "/";
+   if( !newUserDataDir.endsWith("/") )
+      newUserDataDir += "/";
 #else
-	 if( !newUserDataDir.endsWith("\\") && !newUserDataDir.endsWith("/") )
-		 newUserDataDir += "\\";
+   if( !newUserDataDir.endsWith("\\") && !newUserDataDir.endsWith("/") )
+      newUserDataDir += "\\";
 #endif
 
    if( newUserDataDir != Brewtarget::getUserDataDir() )
    {
       // If there are no data files present...
-      if( ! QFileInfo(newUserDataDir + "database.xml").exists() )
+      if( ! QFileInfo(newUserDataDir + "database.sqlite").exists() )
       {
          // ...tell user we will copy old data files to new location.
          QMessageBox::information(this,
@@ -277,10 +306,18 @@ void OptionDialog::saveAndClose()
       }
 
       Brewtarget::userDataDir = newUserDataDir;
+      QMessageBox::information(
+         this,
+         tr("Restart"),
+         tr("Please restart Brewtarget.")
+      );
    }
 
-   if( Brewtarget::mainWindow != 0 )
-      Brewtarget::mainWindow->showChanges(); // Make sure the main window updates.
+   Brewtarget::setOption("mashHopAdjustment", lineEdit_mashHop->text().toDouble() / 100);
+   Brewtarget::setOption("firstWortHopAdjustment", lineEdit_firstWort->text().toDouble() / 100);
+   // Make sure the main window updates.
+   if( Brewtarget::mainWindow() )
+      Brewtarget::mainWindow()->showChanges();
 
    setVisible(false);
 }
@@ -292,11 +329,11 @@ void OptionDialog::cancel()
 
 void OptionDialog::showChanges()
 {
-   // Check the right language button
-   QPushButton* lButton = languageToButtonMap[Brewtarget::getCurrentLanguage()];
-   if( lButton != 0 )
-      lButton->setChecked(true);
-
+   // Set the right language
+   int ndx = ndxToLangCode.indexOf( Brewtarget::getCurrentLanguage() );
+   if( ndx >= 0 )
+      comboBox_lang->setCurrentIndex(ndx);
+   
    // Check the right color formula box.
    switch( Brewtarget::colorFormula )
    {
@@ -381,4 +418,24 @@ void OptionDialog::showChanges()
 
    // Data directory
    lineEdit_dbDir->setText(Brewtarget::getUserDataDir());
+
+   // The IBU modifications. These will all be calculated from a 60 min boil. This is gonna get confusing.
+   lineEdit_mashHop->setText( Brewtarget::displayAmount(Brewtarget::option("mashHopAdjustment",0).toDouble()*100,0,0) );
+   lineEdit_firstWort->setText( Brewtarget::displayAmount(Brewtarget::option("firstWortHopAdjustment", 1.10).toDouble()*100,0,0) );
+
 }
+
+void OptionDialog::changeEvent(QEvent* e)
+{
+   switch( e->type() )
+   {
+      case QEvent::LanguageChange:
+         retranslate();
+         e->accept();
+         break;
+      default:
+         QDialog::changeEvent(e);
+         break;
+   }
+}
+
