@@ -1,6 +1,6 @@
 /*
  * OptionDialog.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,30 +26,50 @@ class OptionDialog;
 #include <QAbstractButton>
 #include <QMap>
 #include <QString>
+#include <QVector>
+#include <QIcon>
 #include "ui_optionsDialog.h"
 #include "unit.h"
 
+/*!
+ * \class OptionDialog
+ * \author Philip G. Lee
+ *
+ * \brief View/controller dialog to manage options.
+ */
 class OptionDialog : public QDialog, public Ui::optionsDialog
 {
    Q_OBJECT
 public:
+   //! \brief Default constructor.
    OptionDialog(QWidget *parent=0);
 
 public slots:
+   //! \brief Show the dialog.
    void show();
+   //! \brief Save the options and close the dialog.
    void saveAndClose();
+   //! \brief Close dialog without saving options.
    void cancel();
+   //! \brief Pop up a dialog to choose the data directory.
    void setDataDir();
+   //! \brief Reset data directory to default.
    void defaultDataDir();
    
+protected:
+   
+   //! \brief Reimplemented from QWidget.
+   virtual void changeEvent(QEvent* e);
+   
 private:
+   // Update UI strings according to current language.
+   void retranslate();
+   // Update dialog with current options.
    void showChanges();
-   QButtonGroup *languageGroup;
    QButtonGroup *colorGroup, *ibuGroup;
    QButtonGroup *weightGroup, *volumeGroup, *tempGroup, *gravGroup, *colorUnitGroup;
-   //! This map makes correspondences from two-letter language codes ("es" for spanish) to their push buttons.
-   QMap<QString,QPushButton*> languageToButtonMap;
+   QStringList ndxToLangCode;
+   QVector<QIcon> langIcons;
 };
 
 #endif   /* _OPTIONDIALOG_H */
-
