@@ -1,6 +1,6 @@
 /*
  * PitchDialog.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2010-2011.
+ * (rocketman768@gmail.com), 2010-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,12 @@ class PitchDialog;
 #include <QWidget>
 #include "ui_pitchDialog.h"
 
+/*!
+ * \class PitchDialog
+ * \author Philip G. Lee
+ *
+ * \brief Dialog to calculate how much yeast to pitch.
+ */
 class PitchDialog : public QDialog, public Ui::pitchDialog
 {
    Q_OBJECT
@@ -33,9 +39,16 @@ public:
    PitchDialog(QWidget* parent=0);
    ~PitchDialog();
 
+   //! \brief Set the wort volume in liters.
+   void setWortVolume_l(double volume);
+   //! \brief Set the wort gravity in 20C/20C SG.
+   void setWortGravity(double sg);
+   
 public slots:
    void calculate();
    void updateShownPitchRate(int percent);
+   void toggleViabilityFromDate(int state);
+   void updateViabilityFromDate(QDate date);
 
 private:
 
