@@ -1,6 +1,6 @@
 /*
 * RecipeExtrasDialog.h is part of Brewtarget, and is Copyright Philip G. Lee
-* (rocketman768@gmail.com), 2010-2011.
+* (rocketman768@gmail.com), 2010-2013.
 *
 * Brewtarget is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -23,17 +23,27 @@ class RecipeExtrasDialog;
 
 #include <QDialog>
 #include <QWidget>
+#include <QMetaProperty>
+#include <QVariant>
 #include "ui_recipeExtrasDialog.h"
-#include "observable.h"
-#include "recipe.h"
 
-class RecipeExtrasDialog : public QDialog, public Ui::recipeExtrasDialog, public Observer
+// Forward declarations.
+class Recipe;
+
+/*!
+ * \class RecipeExtrasDialog
+ * \author Philip G. Lee
+ *
+ * \brief View/controller dialog for editing "extra" fields of the recipe.
+ */
+class RecipeExtrasDialog : public QDialog, public Ui::recipeExtrasDialog
 {
    Q_OBJECT
 
 public:
    RecipeExtrasDialog(QWidget* parent=0);
    virtual ~RecipeExtrasDialog() {}
+   //! Set the recipe to view/edit.
    void setRecipe(Recipe* rec);
 
 public slots:
@@ -54,13 +64,12 @@ public slots:
    void updateNotes();
 
    void saveAndQuit();
+   void changed(QMetaProperty, QVariant);
 
 private:
    Recipe* recObs;
 
-   void showChanges();
-
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer
+   void showChanges(QMetaProperty* prop = 0);
 };
 
 #endif // RECIPEEXTRASDIALOG_H
