@@ -1,6 +1,6 @@
 /*
 * RecipeExtrasWidget.h is part of Brewtarget, and is Copyright Philip G. Lee
-* (rocketman768@gmail.com), 2011.
+* (rocketman768@gmail.com), 2011-2013.
 *
 * Brewtarget is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -23,20 +23,30 @@ class RecipeExtrasWidget;
 
 #include <QDialog>
 #include <QWidget>
+#include <QMetaProperty>
+#include <QVariant>
 #include "ui_recipeExtrasWidget.h"
-#include "observable.h"
-#include "recipe.h"
 
-class RecipeExtrasWidget : public QWidget, public Ui::recipeExtrasWidget, public Observer
+// Forward declarations
+class Recipe;
+
+/*!
+ * \class RecipeExtrasWidget
+ * \author Philip G. Lee
+ *
+ * \brief View/controller widget for modifying "extra" recipe fields.
+ */
+class RecipeExtrasWidget : public QWidget, public Ui::recipeExtrasWidget
 {
    Q_OBJECT
 
 public:
    RecipeExtrasWidget(QWidget* parent=0);
    virtual ~RecipeExtrasWidget() {}
+   //! \brief Set the recipe to view/edit.
    void setRecipe(Recipe* rec);
 
-public slots:
+private slots:
    void updateBrewer();
    void updateBrewerAsst();
    void updateTasteRating();
@@ -48,19 +58,21 @@ public slots:
    void updateTertiaryTemp();
    void updateAge();
    void updateAgeTemp();
-   void updateDate();
+   void updateDate(const QDate &date = QDate());
    void updateCarbonation();
    void updateTasteNotes();
    void updateNotes();
 
+   void changeRatings(int rating);
    void saveAll();
 
+   void changed(QMetaProperty,QVariant);
+   
 private:
-   Recipe* recObs;
+   Recipe* recipe;
+   bool ratingChanged;
 
-   void showChanges();
-
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer
+   void showChanges(QMetaProperty* prop = 0);
 };
 
 #endif // RECIPEEXTRASWIDGET_H
