@@ -1,6 +1,6 @@
 /*
  * SIVolumeUnitSystem.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ SIVolumeUnitSystem::SIVolumeUnitSystem()
 {
 }
 
-QString SIVolumeUnitSystem::displayAmount( double amount, Unit* units )
+QString SIVolumeUnitSystem::displayAmount( double amount, Unit* units, unitScale scale )
 {
    QString SIUnitName = units->getSIUnitName();
    double SIAmount = units->toSI( amount );
@@ -36,12 +36,22 @@ QString SIVolumeUnitSystem::displayAmount( double amount, Unit* units )
    // Special cases. Make sure the unit isn't null and that we're
    // dealing with volume.
    if( units == 0 || SIUnitName.compare("L") != 0 )
-      return QString("%1").arg(amount, fieldWidth, format, precision);
+      return QString("%L1").arg(amount, fieldWidth, format, precision);
 
-   if( qAbs(SIAmount) < Units::liters->toSI(1.0) ) // If less than 1 L, display mL.
-      ret = QString("%1 %2").arg(Units::milliliters->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::milliliters->getUnitName());
-   else // Otherwise, display liters.
-      ret = QString("%1 %2").arg(Units::liters->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::liters->getUnitName());
+   switch(scale)
+   {
+      case extrasmall:
+         ret = QString("%L1 %2").arg(Units::milliliters->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::milliliters->getUnitName());
+         break;
+      case small:
+         ret = QString("%L1 %2").arg(Units::liters->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::liters->getUnitName());
+         break;
+      default:
+         if( qAbs(SIAmount) < Units::liters->toSI(1.0) ) // If less than 1 L, display mL.
+            ret = QString("%L1 %2").arg(Units::milliliters->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::milliliters->getUnitName());
+         else // Otherwise, display liters.
+            ret = QString("%L1 %2").arg(Units::liters->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::liters->getUnitName());
+   }
 
    return ret;
 }
@@ -77,3 +87,5 @@ Unit* SIVolumeUnitSystem::thicknessUnit()
 {
    return Units::liters;
 }
+
+QString SIVolumeUnitSystem::unitType() { return "SI"; }
