@@ -1,6 +1,6 @@
 /*
  * SIWeightUnitSystem.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ SIWeightUnitSystem::SIWeightUnitSystem()
 {
 }
 
-QString SIWeightUnitSystem::displayAmount( double amount, Unit* units )
+QString SIWeightUnitSystem::displayAmount( double amount, Unit* units, unitScale scale )
 {
    QString SIUnitName = units->getSIUnitName();
    double SIAmount = units->toSI( amount );
@@ -37,14 +37,33 @@ QString SIWeightUnitSystem::displayAmount( double amount, Unit* units )
    // Special cases. Make sure the unit isn't null and that we're
    // dealing with mass.
    if( units == 0 || SIUnitName.compare("kg") != 0 )
-      return QString("%1").arg(amount, fieldWidth, format, precision);
+      return QString("%L1").arg(amount, fieldWidth, format, precision);
 
-   if( absSIAmount < Units::grams->toSI(1.0) ) // If less than a gram, show mg.
-      ret = QString("%1 %2").arg(Units::milligrams->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::milligrams->getUnitName());
-   else if( absSIAmount < Units::kilograms->toSI(1.0) ) // If less than a kg, show g.
-      ret = QString("%1 %2").arg(Units::grams->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::grams->getUnitName());
-   else // Otherwise, show kg.
-      ret = QString("%1 %2").arg(Units::kilograms->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::kilograms->getUnitName());
+   switch(scale)
+   {
+      case extrasmall:
+         ret = QString("%L1 %2").arg(Units::milligrams->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::milligrams->getUnitName());
+         break;
+      case small:
+         ret = QString("%L1 %2").arg(Units::grams->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::grams->getUnitName());
+         break;
+      case medium:
+         ret = QString("%L1 %2").arg(Units::kilograms->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::kilograms->getUnitName());
+         break;
+      default:
+         if( absSIAmount < Units::grams->toSI(1.0) ) // If less than a gram, show mg.
+            ret = QString("%L1 %2")
+                  .arg(Units::milligrams->fromSI(SIAmount), fieldWidth, format, precision)
+                  .arg(Units::milligrams->getUnitName());
+         else if( absSIAmount < Units::kilograms->toSI(1.0) ) // If less than a kg, show g.
+            ret = QString("%L1 %2")
+                  .arg(Units::grams->fromSI(SIAmount), fieldWidth, format, precision)
+                  .arg(Units::grams->getUnitName());
+         else // Otherwise, show kg.
+            ret = QString("%L1 %2")
+               .arg(Units::kilograms->fromSI(SIAmount), fieldWidth, format, precision)
+               .arg(Units::kilograms->getUnitName());
+   }
 
    return ret;
 }
@@ -77,3 +96,5 @@ Unit* SIWeightUnitSystem::thicknessUnit()
 {
    return Units::kilograms;
 }
+
+QString SIWeightUnitSystem::unitType() { return "SI"; }
