@@ -1,6 +1,6 @@
 /*
 * ScaleRecipeTool.h is part of Brewtarget, and is Copyright Philip G. Lee
-* (rocketman768@gmail.com), 2009-2011.
+* (rocketman768@gmail.com), 2009-2013.
 *
 * Brewtarget is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -24,10 +24,18 @@ class ScaleRecipeTool;
 #include "ui_scaleRecipeTool.h"
 #include <QDialog>
 #include <QWidget>
-#include "recipe.h"
 #include <QAbstractButton>
 #include <QButtonGroup>
 
+// Forward declarations
+class Recipe;
+
+/*!
+ * \class ScaleRecipeTool
+ * \author Philip G. Lee
+ *
+ * \brief Controller class that scales a recipe's ingredients.
+ */
 class ScaleRecipeTool : public QDialog, public Ui::scaleRecipeTool
 {
    Q_OBJECT
