@@ -1,6 +1,6 @@
 /*
  * StyleEditor.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,16 +22,27 @@
 class StyleEditor;
 
 #include <QDialog>
+#include <QMetaProperty>
+#include <QVariant>
 #include "ui_styleEditor.h"
-#include "style.h"
-#include "observable.h"
 
-class StyleEditor : public QDialog, public Ui::styleEditor, public Observer
+// Forward declarations.
+class Style;
+class StyleListModel;
+class StyleSortFilterProxyModel;
+
+/*!
+ * \class StyleEditor
+ * \author Philip G. Lee
+ *
+ * \brief View/controller dialog to modify styles.
+ */
+class StyleEditor : public QDialog, public Ui::styleEditor
 {
    Q_OBJECT
 
 public:
-   StyleEditor( QWidget *parent=0 );
+   StyleEditor( QWidget *parent=0, bool singleSyleEditor = false );
    virtual ~StyleEditor() {}
    void setStyle( Style* s );
 
@@ -43,12 +54,13 @@ public slots:
    void clearAndClose();
 
    void styleSelected( const QString& text );
-
+   void changed(QMetaProperty,QVariant);
 private:
    Style* obsStyle;
-
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer
-   void showChanges();
+   StyleListModel* styleListModel;
+   StyleSortFilterProxyModel* styleProxyModel;
+   
+   void showChanges(QMetaProperty* prop = 0);
 };
 
 #endif   /* _STYLEEDITOR_H */
