@@ -1,6 +1,6 @@
 /*
  * TimeUnitSystem.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ TimeUnitSystem::TimeUnitSystem()
 {
 }
 
-QString TimeUnitSystem::displayAmount( double amount, Unit* units )
+QString TimeUnitSystem::displayAmount( double amount, Unit* units, unitScale scale )
 {
    QString SIUnitName = units->getSIUnitName();
    double SIAmount = units->toSI( amount );
@@ -36,16 +36,16 @@ QString TimeUnitSystem::displayAmount( double amount, Unit* units )
    // Special cases. Make sure the unit isn't null and that we're
    // dealing with time.
    if( units == 0 || SIUnitName.compare("min") != 0 )
-      return QString("%1").arg(amount, fieldWidth, format, precision);
+      return QString("%L1").arg(amount, fieldWidth, format, precision);
 
    if( absSIAmount < Units::minutes->toSI(1.0) ) // Less than a minute, show seconds.
-      ret = QString("%1 %2").arg(Units::seconds->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::seconds->getUnitName());
+      ret = QString("%L1 %2").arg(Units::seconds->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::seconds->getUnitName());
    else if( absSIAmount < Units::hours->toSI(2.0) ) // Less than two hours, show minutes.
-      ret = QString("%1 %2").arg(Units::minutes->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::minutes->getUnitName());
+      ret = QString("%L1 %2").arg(Units::minutes->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::minutes->getUnitName());
    else if( absSIAmount < Units::days->toSI(1.0) )// Show hours.
-      ret = QString("%1 %2").arg(Units::hours->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::hours->getUnitName());
+      ret = QString("%L1 %2").arg(Units::hours->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::hours->getUnitName());
    else
-      ret = QString("%1 %2").arg(Units::days->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::days->getUnitName());
+      ret = QString("%L1 %2").arg(Units::days->fromSI(SIAmount), fieldWidth, format, precision).arg(Units::days->getUnitName());
 
    return ret;
 }
@@ -73,3 +73,4 @@ void TimeUnitSystem::ensureMapIsSetup()
    isMapSetup = true;
 }
 
+QString TimeUnitSystem::unitType() { return "entropy"; }
