@@ -1,6 +1,6 @@
 /*
  * TimeUnitSystem.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,9 +28,10 @@ class TimeUnitSystem : public UnitSystem
 {
 public:
    TimeUnitSystem();
-   QString displayAmount( double amount, Unit* units = 0 ); /* Inherited from UnitSystem */
+   QString displayAmount( double amount, Unit* units = 0, unitScale scale = noScale ); /* Inherited from UnitSystem */
    double qstringToSI( QString qstr ); /* Inherited from UnitSystem */
    Unit* thicknessUnit(){ return 0; }
+   QString unitType();
 
 private:
    static void ensureMapIsSetup();
