@@ -1,6 +1,6 @@
 /*
  * TimerWidget.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,32 +24,43 @@
 #include <iostream>
 #include "brewtarget.h"
 
-TimerWidget::TimerWidget(QWidget* parent) : QWidget(parent)
+TimerWidget::TimerWidget(QWidget* parent)
+   : QWidget(parent),
+     hours(0),
+     minutes(0),
+     seconds(0),
+     start(true),
+     timer(new QTimer(this)),
+     flashTimer(new QTimer(this)),
+     paletteOld(),
+     paletteNew(),
+#if !defined(NO_PHONON)
+     mediaObject(new Phonon::MediaObject(this)),
+     audioOutput(new Phonon::AudioOutput(Phonon::MusicCategory, this)),
+#endif
+     oldColors(true)
 {
    setupUi(this);
 
-   hours = 0;
-   minutes = 0;
-   seconds = 0;
-   start = true;
-   oldColors = true;
-   timer = new QTimer();
-   flashTimer = new QTimer();
    timer->setInterval(1000); // One second between timeouts.
    flashTimer->setInterval(500);
 
    // PlaceholderText only exists in Qt 4.7 or greater.
    //lineEdit->setPlaceholderText( tr("HH:MM:SS") );
 
-   #if !defined(NO_PHONON)
+#if !defined(NO_PHONON)
+
+   mediaObject->setTransitionTime(0);
+   mediaObject->setPrefinishMark(10); // 10 ms.
+   Phonon::createPath(mediaObject, audioOutput);
+
+   // The following signal is emitted when we are almost at the end of the sound.
+   // The slot re-queues the same song, so we get a loop. 
+   // This needs to be inside the ifdef -- connecting the mediaObject outside
+   // causes a compile error
+   connect( mediaObject, SIGNAL(prefinishMarkReached(qint32)), this, SLOT(doReplay(qint32)) );
    
-    mediaObject = new Phonon::MediaObject(this);
-    mediaObject->setTransitionTime(0);
-    mediaObject->setPrefinishMark(10); // 10 ms.
-    audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory, this);
-    Phonon::createPath(mediaObject, audioOutput);
-   
-   #endif
+#endif
 
    paletteOld = lcdNumber->palette();
    paletteNew = QPalette(paletteOld);
@@ -63,16 +74,12 @@ TimerWidget::TimerWidget(QWidget* parent) : QWidget(parent)
    connect( pushButton_set, SIGNAL(clicked()), this, SLOT(setTimer()) );
    connect( pushButton_startStop, SIGNAL(clicked()), this, SLOT(startStop()) );
    connect( pushButton_sound, SIGNAL(clicked()), this, SLOT(getSound()) );
-   // The following signal is emitted when we are almost at the end of the sound.
-   // The slot re-queues the same song, so we get a loop.
-   connect( mediaObject, SIGNAL(prefinishMarkReached(qint32)), this, SLOT(doReplay(qint32)) );
 
    showChanges();
 }
 
 TimerWidget::~TimerWidget()
 {
-   delete timer;
 }
 
 void TimerWidget::doReplay(qint32 /*msecToEnd*/)
@@ -89,6 +96,7 @@ void TimerWidget::getSound()
    #if !defined(NO_PHONON)
    QString soundFile = QFileDialog::getOpenFileName( qobject_cast<QWidget*>(this), tr("Open Sound"), soundsDir.exists() ? soundsDir.canonicalPath() : "", tr("Audio Files (*.wav *.ogg *.mp3 *.aiff)") );
     if (! soundFile.isNull()) {
+      mediaObject->clearQueue();
       mediaObject->setCurrentSource(soundFile);
       pushButton_sound->setCheckable(true); // indicate a sound is loaded
       pushButton_sound->setChecked(true);
@@ -213,7 +221,9 @@ void TimerWidget::startStop()
    else
    {
       timer->stop();
+#if !defined(NO_PHONON)
       mediaObject->stop();
+#endif
       stopFlashing();
       pushButton_startStop->setText(tr("Start"));
       start = true;
