@@ -1,6 +1,6 @@
 /*
  * USWeightUnitSystem.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,9 +26,10 @@ class USWeightUnitSystem : public UnitSystem
 {
 public:
    USWeightUnitSystem();
-   QString displayAmount( double amount, Unit* units = 0 ); /* Inherited from UnitSystem */
+   QString displayAmount( double amount, Unit* units = 0, unitScale scale = noScale ); /* Inherited from UnitSystem */
    double qstringToSI( QString qstr ); /* Inherited from UnitSystem */
    Unit* thicknessUnit(); /* Inherited from UnitSystem */
+   QString unitType();
 
 private:
    static void ensureMapIsSetup();
