@@ -1,6 +1,6 @@
 /*
  * UnitSystem.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2010.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,12 @@ class UnitSystems;
 #include <QRegExp>
 #include "unit.h"
 
+/*!
+ * \class UnitSystem
+ * \author Philip G. Lee
+ *
+ * \brief A unit system handles the display and format of physical quantities.
+ */
 class UnitSystem
 {
 public:
@@ -37,7 +43,7 @@ public:
     * 'amount' of type 'units' in this UnitSystem. This string should also
     * be recognized by qstringToSI()
     */
-   virtual QString displayAmount( double amount, Unit* units = 0 ) = 0;
+   virtual QString displayAmount( double amount, Unit* units = 0, unitScale scale = noScale ) = 0;
 
    /*!
     * qstringToSI() should convert 'qstr' (consisting of a decimal amount,
@@ -53,12 +59,15 @@ public:
     */
    virtual Unit* thicknessUnit() = 0;
 
+   // \brief Returns the name of the unit
+   virtual QString unitType() = 0;
+
    double qstringToSI(QString qstr, Unit* defUnit);
 
 protected:
-   static const int fieldWidth = 0;
-   static const char format = 'f';
-   static const int precision = 3;
+   static const int fieldWidth;
+   static const char format;
+   static const int precision;
    static QMap<QString, Unit*> nameToUnit;
    QRegExp amtUnit;
 
