@@ -1,6 +1,6 @@
 /*
  * WaterEditor.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,11 +20,20 @@
 #define WATEREDITOR_H
 
 #include <QDialog>
+#include <QMetaProperty>
+#include <QVariant>
 #include "ui_waterEditor.h"
-#include "observable.h"
-#include "water.h"
 
-class WaterEditor : public QDialog, public Ui::waterEditor, public Observer
+// Forward declarations.
+class Water;
+
+/*!
+ * \class WaterEditor
+ * \author Philip G. Lee
+ *
+ * \brief View/controller class for modifying water records.
+ */
+class WaterEditor : public QDialog, public Ui::waterEditor
 {
     Q_OBJECT
 public:
@@ -36,11 +45,10 @@ public:
      */
     void setWater(Water* water);
 
-    virtual void notify(Observable *notifier, QVariant info); // From Observer.
-
  public slots:
-    void showChanges();
+    void showChanges(QMetaProperty* prop = 0);
     void saveAndClose();
+    void changed(QMetaProperty,QVariant);
 
 private:
     Water* obs; // Observed water.
