@@ -1,6 +1,6 @@
 /*
  * WaterTableModel.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,43 +25,68 @@ class WaterItemDelegate;
 #include <QAbstractTableModel>
 #include <QWidget>
 #include <QModelIndex>
+#include <QMetaProperty>
 #include <QVariant>
-#include <Qt>
 #include <QItemDelegate>
-#include <QVector>
-#include "water.h"
-#include "observable.h"
-#include "WaterTableWidget.h"
+#include <QList>
+
+// Forward declarations.
+class Water;
+class WaterTableWidget;
+class Recipe;
 
 enum{ WATERNAMECOL, WATERAMOUNTCOL, WATERCALCIUMCOL, WATERBICARBONATECOL,
       WATERSULFATECOL, WATERCHLORIDECOL, WATERSODIUMCOL, WATERMAGNESIUMCOL,
       WATERNUMCOLS /*This one MUST be last*/};
 
-class WaterTableModel : public QAbstractTableModel, public MultipleObserver
+/*!
+ * \class WaterTableModel
+ * \author Philip G. Lee
+ *
+ * \brief Table model for waters.
+ */
+class WaterTableModel : public QAbstractTableModel
 {
    Q_OBJECT
 
 public:
    WaterTableModel(WaterTableWidget* parent=0);
    virtual ~WaterTableModel() {}
-   void addWater(Water* water);
-   bool removeWater(Water* water); // Returns true if "water" is successfully found and removed.
+   void addWaters(QList<Water*> waters);
+   void observeRecipe(Recipe* rec);
+   void observeDatabase(bool val);
    void removeAll();
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer via MultipleObserver.
 
-   // Inherit the following from QAbstractItemModel via QAbstractTableModel
+   //! Reimplemented from QAbstractTableModel.
    virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
+   //! Reimplemented from QAbstractTableModel.
    virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
+   //! Reimplemented from QAbstractTableModel.
    virtual QVariant data( const QModelIndex& index, int role = Qt::DisplayRole ) const;
+   //! Reimplemented from QAbstractTableModel.
    virtual QVariant headerData( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
+   //! Reimplemented from QAbstractTableModel.
    virtual Qt::ItemFlags flags(const QModelIndex& index ) const;
+   //! Reimplemented from QAbstractTableModel.
    virtual bool setData( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
 
+public slots:
+   void changed(QMetaProperty,QVariant);
+   void addWater(Water* water);
+   void removeWater(Water* water);
+   
 private:
-   QVector<Water*> waterObs;
+   QList<Water*> waterObs;
+   Recipe* recObs;
    WaterTableWidget* parentTableWidget;
 };
 
+/*!
+ * \class WaterItemDelegate
+ * \author Philip G. Lee
+ *
+ * Item delegate for water tables.
+ */
 class WaterItemDelegate : public QItemDelegate
 {
    Q_OBJECT
