@@ -1,6 +1,6 @@
 /*
  * YeastDialog.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,21 +24,29 @@ class YeastDialog;
 #include <QWidget>
 #include <QDialog>
 #include <QVariant>
+#include <QMetaProperty>
 #include "ui_yeastDialog.h"
-#include "observable.h"
-#include "database.h"
-class MainWindow;
-#include "YeastEditor.h"
 
-class YeastDialog : public QDialog, public Ui::yeastDialog, public Observer
+// Forward declarations.
+class MainWindow;
+class YeastEditor;
+class YeastEditor;
+class YeastTableModel;
+class YeastSortFilterProxyModel;
+
+/*!
+ * \class YeastDialog
+ * \author Philip G. Lee
+ *
+ * \brief View/controller dialog for displaying all the yeasts in the database.
+ */
+class YeastDialog : public QDialog, public Ui::yeastDialog
 {
    Q_OBJECT
 
 public:
    YeastDialog(MainWindow* parent);
    virtual ~YeastDialog() {}
-   void startObservingDB();
-   virtual void notify(Observable *notifier, QVariant info = QVariant()); // From Observer
 
 public slots:
    void addYeast(const QModelIndex& = QModelIndex());
@@ -47,12 +55,11 @@ public slots:
    void newYeast();
 
 private:
-   Database* dbObs;
    MainWindow* mainWindow;
+   YeastTableModel* yeastTableModel;
+   YeastSortFilterProxyModel* yeastTableProxy;
    YeastEditor* yeastEditor;
-   unsigned int numYeasts;
-
-   void populateTable();
+   int numYeasts;
 };
 
 #endif   /* _YEASTDIALOG_H */
