@@ -1,6 +1,6 @@
 /*
  * YeastEditor.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,28 +21,39 @@
 
 class YeastEditor;
 
+#include <QDialog>
+#include <QMetaProperty>
+#include <QVariant>
 #include "ui_yeastEditor.h"
-#include "yeast.h"
-#include "observable.h"
 
-class YeastEditor : public QDialog, private Ui::yeastEditor, Observer
+// Forward declarations.
+class Yeast;
+
+/*!
+ * \class YeastEditor
+ * \author Philip G. Lee
+ *
+ * \brief View/controller dialog for modifying yeasts.
+ */
+class YeastEditor : public QDialog, private Ui::yeastEditor
 {
    Q_OBJECT
 
 public:
    YeastEditor( QWidget *parent=0 );
    virtual ~YeastEditor() {}
+   //! Set the yeast we want to modify.
    void setYeast( Yeast* y );
 
 public slots:
    void save();
    void clearAndClose();
-
+   void changed(QMetaProperty,QVariant);
+   
 private:
    Yeast* obsYeast;
 
-   virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer
-   void showChanges();
+   void showChanges(QMetaProperty* prop = 0);
 };
 
 #endif   /* _YEASTEDITOR_H */
