@@ -1,6 +1,7 @@
 /*
  * YeastSortFilterProxyModel.cpp is part of Brewtarget, and is Copyright Mik
- * Firestone (mikfire@gmail.com), 2010-2011.
+ * Firestone (mikfire@gmail.com) and Philip G. Lee (rocketman768@gmail.com),
+ * 2010-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,17 +24,25 @@ class YeastSortFilterProxyModel;
 
 #include <QSortFilterProxyModel>
 
+/*!
+ * \class YeastSortFilterProxyModel
+ * \author Mik Firestone
+ *
+ * \brief Proxy model for sorting yeasts.
+ */
 class YeastSortFilterProxyModel : public QSortFilterProxyModel
 {
    Q_OBJECT
 
 public:
-   YeastSortFilterProxyModel(QObject *parent = 0);
-//   QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex());
+   YeastSortFilterProxyModel(QObject *parent = 0, bool filt = true);
 
 protected:
    bool lessThan(const QModelIndex &left, const QModelIndex &right) const;
+   bool filterAcceptsRow( int source_row, const QModelIndex &source_parent) const;
 
+private:
+   bool filter;
 };
 
 #endif
