@@ -1,6 +1,6 @@
 /*
  * equipment.cpp is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,134 +16,55 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <iostream>
-#include <string>
 #include <QVector>
-#include "equipment.h"
-#include "brewtarget.h"
 #include <QDomElement>
 #include <QDomText>
 #include <QObject>
+#include "equipment.h"
+#include "brewtarget.h"
 #include "HeatCalculations.h"
 
+QHash<QString,QString> Equipment::tagToProp = Equipment::tagToPropHash();
+
+QHash<QString,QString> Equipment::tagToPropHash()
+{
+   QHash<QString,QString> propHash;
+   
+   propHash["NAME"] = "name";
+   propHash["BOIL_SIZE"] = "boilSize_l";
+   propHash["BATCH_SIZE"] = "batchSize_l";
+   propHash["TUN_VOLUME"] = "tunVolume_l";
+   propHash["TUN_WEIGHT"] = "tunWeight_kg";
+   propHash["TUN_SPECIFIC_HEAT"] = "tunSpecificHeat_calGC";
+   propHash["TOP_UP_WATER"] = "topUpWater_l";
+   propHash["TRUB_CHILLER_LOSS"] = "trubChillerLoss_l";
+   propHash["EVAP_RATE"] = "evapRate_pctHr";
+   propHash["REAL_EVAP_RATE"] = "evapRate_lHr";
+   propHash["BOIL_TIME"] = "boilTime_min";
+   propHash["CALC_BOIL_VOLUME"] = "calcBoilVolume";
+   propHash["LAUTER_DEADSPACE"] = "lauterDeadspace_l";
+   propHash["TOP_UP_KETTLE"] = "topUpKettle_l";
+   propHash["HOP_UTILIZATION"] = "hopUtilization_pct";
+   propHash["NOTES"] = "notes";
+   propHash["ABSORPTION"] = "grainAbsorption_LKg";
+   propHash["BOILING_POINT"] = "boilingPoint_c";
+   
+   return propHash;
+}
+
 bool operator<(Equipment &e1, Equipment &e2)
 {
-   return e1.name < e2.name;
+   return e1.name() < e2.name();
 }
 
 bool operator==(Equipment &e1, Equipment &e2)
 {
-   return e1.name == e2.name;
-}
-
-void Equipment::toXml(QDomDocument& doc, QDomNode& parent)
-{
-   QDomElement equipNode;
-   QDomElement tmpNode;
-   QDomText tmpText;
-   
-   equipNode = doc.createElement("EQUIPMENT");
-   
-   tmpNode = doc.createElement("NAME");
-   tmpText = doc.createTextNode(name);
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("VERSION");
-   tmpText = doc.createTextNode(text(version));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("BOIL_SIZE");
-   tmpText = doc.createTextNode(text(boilSize_l));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("BATCH_SIZE");
-   tmpText = doc.createTextNode(text(batchSize_l));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("TUN_VOLUME");
-   tmpText = doc.createTextNode(text(tunVolume_l));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("TUN_WEIGHT");
-   tmpText = doc.createTextNode(text(tunWeight_kg));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("TUN_SPECIFIC_HEAT");
-   tmpText = doc.createTextNode(text(tunSpecificHeat_calGC));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("TOP_UP_WATER");
-   tmpText = doc.createTextNode(text(topUpWater_l));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("TRUB_CHILLER_LOSS");
-   tmpText = doc.createTextNode(text(trubChillerLoss_l));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("EVAP_RATE");
-   tmpText = doc.createTextNode(text(evapRate_pctHr));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("REAL_EVAP_RATE");
-   tmpText = doc.createTextNode(text(evapRate_lHr));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-
-   tmpNode = doc.createElement("BOIL_TIME");
-   tmpText = doc.createTextNode(text(boilTime_min));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("CALC_BOIL_VOLUME");
-   tmpText = doc.createTextNode(text(calcBoilVolume));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("LAUTER_DEADSPACE");
-   tmpText = doc.createTextNode(text(lauterDeadspace_l));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("TOP_UP_KETTLE");
-   tmpText = doc.createTextNode(text(topUpKettle_l));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("HOP_UTILIZATION");
-   tmpText = doc.createTextNode(text(hopUtilization_pct));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("NOTES");
-   tmpText = doc.createTextNode(notes);
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-
-   // My extensions below
-   tmpNode = doc.createElement("ABSORPTION");
-   tmpText = doc.createTextNode(text(absorption_LKg));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   
-   tmpNode = doc.createElement("BOILING_POINT");
-   tmpText = doc.createTextNode(text(boilingPoint_c));
-   tmpNode.appendChild(tmpText);
-   equipNode.appendChild(tmpNode);
-   parent.appendChild(equipNode);
+   return e1.name() == e2.name();
 }
 
 //=============================CONSTRUCTORS=====================================
 
+/*
 void Equipment::setDefaults()
 {
    name = "";
@@ -165,34 +86,19 @@ void Equipment::setDefaults()
    absorption_LKg = HeatCalculations::absorption_LKg;
    boilingPoint_c = 100.0;
 }
+*/
 
 Equipment::Equipment()
+   : BeerXMLElement()
 {
-   setDefaults();
 }
 
-Equipment::Equipment(const QDomNode& equipmentNode)
+Equipment::Equipment( Equipment const& other )
+   : BeerXMLElement(other)
 {
-   fromNode(equipmentNode);
-}
-
-Equipment::Equipment( Equipment* other )
-{
-   if( other == 0 )
-   {
-      setDefaults();
-      return;
-   }
-
-   QDomDocument doc;
-   QDomElement root = doc.createElement("root");
-   QDomNodeList list;
-
-   other->toXml(doc, root);
-
-   fromNode(root.firstChild());
 }
 
+/*
 void Equipment::fromNode(const QDomNode& equipmentNode)
 {
    QDomNode node, child;
@@ -304,13 +210,14 @@ void Equipment::fromNode(const QDomNode& equipmentNode)
    if( ! hasRealEvapRate )
       setEvapRate_lHr( evapRate_pctHr/(double)100 * boilSize_l );
 }
+*/
 
 //============================"SET" METHODS=====================================
 
 void Equipment::setName( const QString &var )
 {
-   name = QString(var);
-   hasChanged();
+   set( "name", "name", var );
+   emit changedName(var);
 }
 
 void Equipment::setBoilSize_l( double var )
@@ -318,13 +225,12 @@ void Equipment::setBoilSize_l( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: boil size negative: %1").arg(var) );
-      boilSize_l = 0;
-      hasChanged();
+      return;
    }
    else
    {
-      boilSize_l = var;
-      hasChanged();
+      set("boilSize_l", "boil_size", var);
+      emit changedBoilSize_l(var);
    }
 }
 
@@ -333,14 +239,11 @@ void Equipment::setBatchSize_l( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: batch size negative: %1").arg(var) );
-      batchSize_l = 0;
-      hasChanged();
-      doCalculations();
+      return;
    }
    else
    {
-      batchSize_l = var;
-      hasChanged();
+      set("batchSize_l", "batch_size", var);
       doCalculations();
    }
 }
@@ -350,13 +253,11 @@ void Equipment::setTunVolume_l( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: tun volume negative: %1").arg(var) );
-      tunVolume_l = 0;
-      hasChanged();
+      return;
    }
    else
    {
-      tunVolume_l = var;
-      hasChanged();
+      set("tunVolume_l", "tun_volume", var);
    }
 }
 
@@ -365,13 +266,11 @@ void Equipment::setTunWeight_kg( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: tun weight negative: %1").arg(var) );
-      tunWeight_kg = 0;
-      hasChanged();
+      return;
    }
    else
    {
-      tunWeight_kg = var;
-      hasChanged();
+      set("tunWeight_kg", "tun_weight", var);
    }
 }
 
@@ -380,13 +279,11 @@ void Equipment::setTunSpecificHeat_calGC( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: tun sp heat negative: %1").arg(var) );
-      tunSpecificHeat_calGC = 0;
-      hasChanged();
+      return;
    }
    else
    {
-      tunSpecificHeat_calGC = var;
-      hasChanged();
+      set("tunSpecificHeat_calGC", "tun_specific_heat", var);
    }
 }
 
@@ -395,14 +292,11 @@ void Equipment::setTopUpWater_l( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: top up water negative: %1").arg(var) );
-      topUpWater_l = 0;
-      hasChanged();
-      doCalculations();
+      return;
    }
    else
    {
-      topUpWater_l = var;
-      hasChanged();
+      set("topUpWater_l", "top_up_water", var);
       doCalculations();
    }
 }
@@ -412,14 +306,11 @@ void Equipment::setTrubChillerLoss_l( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: trub chiller loss negative: %1").arg(var) );
-      trubChillerLoss_l = 0;
-      hasChanged();
-      doCalculations();
+      return;
    }
    else
    {
-      trubChillerLoss_l = var;
-      hasChanged();
+      set("trubChillerLoss_l", "trub_chiller_loss", var);
       doCalculations();
    }
 }
@@ -429,14 +320,12 @@ void Equipment::setEvapRate_pctHr( double var )
    if( var < 0.0 || var > 100.0)
    {
       Brewtarget::logW( QString("Equipment: 0 < evap rate < 100: %1").arg(var) );
-      evapRate_pctHr = 0;
-      hasChanged();
-      doCalculations();
+      return;
    }
    else
    {
-      evapRate_pctHr = var;
-      hasChanged();
+      set("evapRate_pctHr", "evap_rate", var);
+      set("evapRate_lHr", "real_evap_rate", var/100.0 * batchSize_l() ); // We always use this one, so set it.
       doCalculations();
    }
 }
@@ -446,14 +335,12 @@ void Equipment::setEvapRate_lHr( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: evap rate negative: %1").arg(var) );
-      evapRate_lHr = 0;
-      hasChanged();
-      doCalculations();
+      return;
    }
    else
    {
-      evapRate_lHr = var;
-      hasChanged();
+      set("evapRate_lHr", "real_evap_rate", var);
+      setEvapRate_pctHr( var/batchSize_l() * 100.0 ); // We don't use it, but keep it current.
       doCalculations();
    }
 }
@@ -463,22 +350,19 @@ void Equipment::setBoilTime_min( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: boil time negative: %1").arg(var) );
-      boilTime_min = 0;
-      hasChanged();
-      doCalculations();
+      return;
    }
    else
    {
-      boilTime_min = var;
-      hasChanged();
+      set("boilTime_min", "boil_time", var);
+      emit changedBoilTime_min(var);
       doCalculations();
    }
 }
 
 void Equipment::setCalcBoilVolume( bool var )
 {
-   calcBoilVolume = var;
-   hasChanged();
+   set("calcBoilVolume", "calc_boil_volume", var);
    if( var )
       doCalculations();
 }
@@ -488,13 +372,11 @@ void Equipment::setLauterDeadspace_l( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: deadspace negative: %1").arg(var) );
-      lauterDeadspace_l = 0;
-      hasChanged();
+      return;
    }
    else
    {
-      lauterDeadspace_l = var;
-      hasChanged();
+      set("lauterDeadspace_l", "lauter_deadspace", var);
    }
 }
 
@@ -503,35 +385,30 @@ void Equipment::setTopUpKettle_l( double var )
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: top up kettle negative: %1").arg(var) );
-      topUpKettle_l = 0;
-      hasChanged();
+      return;
    }
    else
    {
-      topUpKettle_l = var;
-      hasChanged();
+      set("topUpKettle_l", "top_up_kettle", var);
    }
 }
 
 void Equipment::setHopUtilization_pct( double var )
 {
-   if( var < 0.0 || var > 100.0 )
+   if( var < 0.0 )
    {
-      Brewtarget::logW( QString("Equipment: 0 < hop utilization < 100: %1").arg(var) );
-      hopUtilization_pct = 20;
-      hasChanged();
+      Brewtarget::logW( QString("Equipment: 0 < hop utilization: %1").arg(var) );
+      return;
    }
    else
    {
-      hopUtilization_pct = var;
-      hasChanged();
+      set("hopUtilization_pct", "hop_utilization", var);
    }
 }
 
 void Equipment::setNotes( const QString &var )
 {
-   notes = QString(var);
-   hasChanged();
+   set("notes", "notes", var);
 }
 
 void Equipment::setGrainAbsorption_LKg(double var)
@@ -539,14 +416,12 @@ void Equipment::setGrainAbsorption_LKg(double var)
    if( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: absorption < 0: %1").arg(var) );
-      absorption_LKg = HeatCalculations::absorption_LKg;
+      return;
    }
    else
    {
-      absorption_LKg = var;
+      set("absorption_LKg", "absorption", var);
    }
-
-   hasChanged();
 }
 
 void Equipment::setBoilingPoint_c(double var)
@@ -554,128 +429,118 @@ void Equipment::setBoilingPoint_c(double var)
    if ( var < 0.0 )
    {
       Brewtarget::logW( QString("Equipment: boiling point of water < 0: %1").arg(var));
-      boilingPoint_c = 100.0;
+      return;
    }
    else 
    {
-      boilingPoint_c = var;
+      set("boilingPoint_c", "boiling_point", var);
    }
 }
 
 //============================"GET" METHODS=====================================
 
-QString Equipment::getName() const
+QString Equipment::name() const
 {
-   return name;
+   return get("name").toString();
 }
 
-double Equipment::getBoilSize_l() const
+double Equipment::boilSize_l() const
 {
-   return boilSize_l;
+   return get("boil_size").toDouble();
 }
 
-double Equipment::getBatchSize_l() const
+double Equipment::batchSize_l() const
 {
-   return batchSize_l;
+   return get("batch_size").toDouble();
 }
 
-double Equipment::getTunVolume_l() const
+double Equipment::tunVolume_l() const
 {
-   return tunVolume_l;
+   return get("tun_volume").toDouble();
 }
 
-double Equipment::getTunWeight_kg() const
+double Equipment::tunWeight_kg() const
 {
-   return tunWeight_kg;
+   return get("tun_weight").toDouble();
 }
 
-double Equipment::getTunSpecificHeat_calGC() const
+double Equipment::tunSpecificHeat_calGC() const
 {
-   return tunSpecificHeat_calGC;
+   return get("tun_specific_heat").toDouble();
 }
 
-double Equipment::getTopUpWater_l() const
+double Equipment::topUpWater_l() const
 {
-   return topUpWater_l;
+   return get("top_up_water").toDouble();
 }
 
-double Equipment::getTrubChillerLoss_l() const
+double Equipment::trubChillerLoss_l() const
 {
-   return trubChillerLoss_l;
+   return get("trub_chiller_loss").toDouble();
 }
 
-double Equipment::getEvapRate_pctHr() const
+double Equipment::evapRate_pctHr() const
 {
-   return evapRate_pctHr;
+   return get("evap_rate").toDouble();
 }
 
-double Equipment::getEvapRate_lHr() const
+double Equipment::evapRate_lHr() const
 {
-   return evapRate_lHr;
+   return get("real_evap_rate").toDouble();
 }
 
-double Equipment::getBoilTime_min() const
+double Equipment::boilTime_min() const
 {
-   return boilTime_min;
+   return get("boil_time").toDouble();
 }
 
-bool Equipment::getCalcBoilVolume() const
+bool Equipment::calcBoilVolume() const
 {
-   return calcBoilVolume;
+   return get("calc_boil_volume").toBool();
 }
 
-double Equipment::getLauterDeadspace_l() const
+double Equipment::lauterDeadspace_l() const
 {
-   return lauterDeadspace_l;
+   return get("lauter_deadspace").toDouble();
 }
 
-double Equipment::getTopUpKettle_l() const
+double Equipment::topUpKettle_l() const
 {
-   return topUpKettle_l;
+   return get("top_up_kettle").toDouble();
 }
 
-double Equipment::getHopUtilization_pct() const
+double Equipment::hopUtilization_pct() const
 {
-   return hopUtilization_pct;
+   return get("hop_utilization").toDouble();
 }
 
-QString Equipment::getNotes() const
+QString Equipment::notes() const
 {
-   return notes;
+   return get("notes").toString();
 }
 
-double Equipment::getGrainAbsorption_LKg()
+double Equipment::grainAbsorption_LKg()
 {
-   return absorption_LKg;
+   return get("absorption").toDouble();
 }
 
-double Equipment::getBoilingPoint_c() const
+double Equipment::boilingPoint_c() const
 {
-   return boilingPoint_c;
+   return get("boiling_point").toDouble();
 }
 
-//TODO: take a look at evapRate_pctHr.
 void Equipment::doCalculations()
 {
    // Only do the calculation if we're asked to.
-   if( ! calcBoilVolume )
+   if( ! calcBoilVolume() )
       return;
-   
-   /* The equation given the BeerXML 1.0 spec was way wrong. */
-   /*
-   boilSize_l =
-      (batchSize_l - topUpWater_l + trubChillerLoss_l)
-      / (1 - (boilTime_min/(double)60) * (evapRate_pctHr/(double)100) );
-   */
 
-   boilSize_l = batchSize_l - topUpWater_l + trubChillerLoss_l + (boilTime_min/(double)60)*evapRate_lHr;
-
-   hasChanged();
+   setBoilSize_l( batchSize_l() - topUpWater_l() + trubChillerLoss_l() + (boilTime_min()/(double)60)*evapRate_lHr());
 }
 
 double Equipment::wortEndOfBoil_l( double kettleWort_l ) const
 {
    //return kettleWort_l * (1 - (boilTime_min/(double)60) * (evapRate_pctHr/(double)100) );
 
-   return kettleWort_l - (boilTime_min/(double)60)*evapRate_lHr;
+   return kettleWort_l - (boilTime_min()/(double)60)*evapRate_lHr();
 }
