@@ -1,6 +1,6 @@
 /*
  * unit.h is part of Brewtarget, and is Copyright Philip G. Lee
- * (rocketman768@gmail.com), 2009-2011.
+ * (rocketman768@gmail.com), 2009-2013.
  *
  * Brewtarget is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ class PoundUnit;
 class OunceUnit;
 class LiterUnit;
 class MilliliterUnit;
+class USBarrelUnit;
 class USGallonUnit;
 class USQuartUnit;
 class USCupUnit;
@@ -44,6 +45,7 @@ class FahrenheitUnit;
 class KelvinUnit;
 
 #include <QString>
+#include <QObject>
 #include <string>
 #include <map>
 #include <QMultiMap>
@@ -59,11 +61,11 @@ enum UnitType
 
 enum iUnitSystem
 {
-    SI,
-    USCustomary,
-    Imperial,
-    ImperialAndUS,
-    Any
+    SI = 0,
+    USCustomary = 1,
+    Imperial = 2,
+    ImperialAndUS = 3,
+    Any = 4
 };
 
 enum TempScale
@@ -73,6 +75,30 @@ enum TempScale
     Kelvin
 };
 
+// DO NOT change noscale's value. Lots of assumptions are made based on this
+// value, both in the code and (more importantly) in the database.
+enum unitScale
+{
+   noScale = -1,
+   extrasmall = 0,
+   small = 1,
+   medium = 2,
+   large = 3,
+   extralarge = 4
+};
+
+enum unitDisplay
+{
+   noUnit       = -1,
+   displaySI    = 0x100,
+   displayUS    = 0x101,
+   displayImp   = 0x102,
+   displaySrm   = 0x200,
+   displayEbc   = 0x201,
+   displaySg    = 0x300,
+   displayPlato = 0x301
+};
+
 inline QString unitSystemToString(iUnitSystem us)
 {
    switch (us)
@@ -95,8 +121,18 @@ inline QString tempScaleToString(TempScale ts)
 }
 // TODO: implement ppm, percent, diastatic power, ibuGalPerLb, gravity, srm, volumes.
 
-class Unit
+/*!
+ * \class Unit
+ * \author Philip G. Lee
+ *
+ * \brief Interface for arbitrary physical units and their formatting.
+ */
+class Unit : public QObject
 {
+   Q_OBJECT
+   Q_ENUMS(unitDisplay)
+   Q_ENUMS(unitScale)
+
    public:
       virtual ~Unit() {}
       virtual double toSI( double amt ) const =0;// { return amt; }
@@ -249,6 +285,24 @@ class MilliliterUnit : public Unit
       QString SIUnitName;
 };
 
+class USBarrelUnit : public Unit
+{
+   public:
+      USBarrelUnit();
+      
+      // Inherited methods.
+      double toSI( double amt ) const;
+      double fromSI( double amt ) const;
+      const QString getUnitName() const { return unitName; }
+      const QString getSIUnitName() const { return SIUnitName; }
+      const int getUnitType() const { return Volume; }
+      const int getUnitOrTempSystem() const { return USCustomary; }
+      
+   private:
+      QString unitName;
+      QString SIUnitName;
+};
+
 class USGallonUnit : public Unit
 {
    public:
@@ -303,6 +357,24 @@ class USCupUnit : public Unit
       QString SIUnitName;
 };
 
+class ImperialBarrelUnit : public Unit
+{
+   public:
+      ImperialBarrelUnit();
+      
+      // Inherited methods.
+      double toSI( double amt ) const;
+      double fromSI( double amt ) const;
+      const QString getUnitName() const { return unitName; }
+      const QString getSIUnitName() const { return SIUnitName; }
+      const int getUnitType() const { return Volume; }
+      const int getUnitOrTempSystem() const { return Imperial; }
+      
+   private:
+      QString unitName;
+      QString SIUnitName;
+};
+
 class ImperialGallonUnit : public Unit
 {
    public:
@@ -612,9 +684,11 @@ public:
    // === Volume ===
    static LiterUnit *liters;
    static MilliliterUnit *milliliters;
+   static USBarrelUnit *us_barrels;
    static USGallonUnit *us_gallons;
    static USQuartUnit *us_quarts;
    static USCupUnit *us_cups;
+   static ImperialBarrelUnit *imperial_barrels;
    static ImperialGallonUnit *imperial_gallons;
    static ImperialQuartUnit *imperial_quarts;
    static ImperialCupUnit *imperial_cups;
